"""Tests for build_jobs_data_basic.py specialized job label builders."""

from collections.abc import Iterable

from ArWikiCats.translations.data_builders.build_jobs_data_basic import (
    _build_military_job_labels,
    _build_painter_job_labels,
    _build_religious_job_labels,
)
from ArWikiCats.translations.data_builders.jobs_defs import GenderedLabelMap


class TestBuildReligiousJobLabels:
    """Tests for _build_religious_job_labels function."""

    def test_combines_religions_and_roles(self) -> None:
        religions: GenderedLabelMap = {
            "christian": {"males": "مسيحي", "females": "مسيحية"},
        }
        roles: GenderedLabelMap = {
            "saints": {"males": "قديسون", "females": "قديسات"},
        }
        result = _build_religious_job_labels(religions, roles)

        assert "christian saints" in result
        assert "قديسون" in result["christian saints"]["males"]
        assert "مسيحية" in result["christian saints"]["females"]

    def test_combines_multiple_pairs(self) -> None:
        religions: GenderedLabelMap = {
            "christian": {"males": "مسيحي", "females": "مسيحية"},
            "islamic": {"males": "إسلامي", "females": "إسلامية"},
        }
        roles: GenderedLabelMap = {
            "saints": {"males": "قديسون", "females": "قديسات"},
            "scholars": {"males": "علماء", "females": "عالمات"},
        }
        result = _build_religious_job_labels(religions, roles)

        assert "christian saints" in result
        assert "christian scholars" in result
        assert "islamic saints" in result
        assert "islamic scholars" in result

    def test_skips_empty_religions(self) -> None:
        religions: GenderedLabelMap = {}
        roles: GenderedLabelMap = {
            "saints": {"males": "قديسون", "females": "قديسات"},
        }
        result = _build_religious_job_labels(religions, roles)

        assert result == {}

    def test_skips_empty_roles(self) -> None:
        religions: GenderedLabelMap = {
            "christian": {"males": "مسيحي", "females": "مسيحية"},
        }
        roles: GenderedLabelMap = {}
        result = _build_religious_job_labels(religions, roles)

        assert result == {}

    def test_skips_empty_gendered_labels(self) -> None:
        religions: GenderedLabelMap = {
            "valid": {"males": "صحيح", "females": "صحيحة"},
            "empty": {"males": "", "females": ""},
        }
        roles: GenderedLabelMap = {
            "saints": {"males": "قديسون", "females": "قديسات"},
        }
        result = _build_religious_job_labels(religions, roles)

        assert "valid saints" in result
        assert "empty saints" not in result


class TestBuildPainterJobLabels:
    """Tests for _build_painter_job_labels function."""

    def test_includes_base_roles(self) -> None:
        painter_styles: GenderedLabelMap = {}
        painter_roles: GenderedLabelMap = {
            "painters": {"males": "رسامون", "females": "رسامات"},
        }
        painter_categories = {}

        result = _build_painter_job_labels(painter_styles, painter_roles, painter_categories)

        assert "painters" in result
        assert result["painters"]["males"] == "رسامون"

    def test_includes_styles_except_history(self) -> None:
        painter_styles: GenderedLabelMap = {
            "impressionist": {"males": "انطباعي", "females": "انطباعية"},
            "history": {"males": "تاريخي", "females": "تاريخية"},
        }
        painter_roles: GenderedLabelMap = {}
        painter_categories = {}

        result = _build_painter_job_labels(painter_styles, painter_roles, painter_categories)

        assert "impressionist" in result
        assert "history" not in result

    def test_creates_style_role_combinations(self) -> None:
        painter_styles: GenderedLabelMap = {
            "symbolist": {"males": "رمزي", "females": "رمزية"},
        }
        painter_roles: GenderedLabelMap = {
            "painters": {"males": "رسامون", "females": "رسامات"},
        }
        painter_categories = {}

        result = _build_painter_job_labels(painter_styles, painter_roles, painter_categories)

        assert "symbolist painters" in result
        assert "رسامون" in result["symbolist painters"]["males"]
        assert "رمزي" in result["symbolist painters"]["males"]

    def test_creates_category_painters_variants(self) -> None:
        painter_styles: GenderedLabelMap = {}
        painter_roles: GenderedLabelMap = {}
        painter_categories = {"french": "فرنسية"}

        result = _build_painter_job_labels(painter_styles, painter_roles, painter_categories)

        assert "french painters" in result
        assert result["french painters"]["males"] == "رسامو فرنسية"

    def test_creates_category_artists_variants(self) -> None:
        painter_styles: GenderedLabelMap = {}
        painter_roles: GenderedLabelMap = {}
        painter_categories = {"modern": "حديثة"}

        result = _build_painter_job_labels(painter_styles, painter_roles, painter_categories)

        assert "modern artists" in result
        assert result["modern artists"]["males"] == "فنانو حديثة"

    def test_skips_empty_categories(self) -> None:
        painter_styles: GenderedLabelMap = {}
        painter_roles: GenderedLabelMap = {}
        painter_categories = {"valid": "صحيح", "empty": ""}

        result = _build_painter_job_labels(painter_styles, painter_roles, painter_categories)

        assert "valid painters" in result
        assert "valid artists" in result
        assert "empty painters" not in result
        assert "empty artists" not in result


class TestBuildMilitaryJobLabels:
    """Tests for _build_military_job_labels function."""

    def test_includes_base_roles(self) -> None:
        military_prefixes: GenderedLabelMap = {}
        military_roles: GenderedLabelMap = {
            "soldiers": {"males": "جنود", "females": "جنوديات"},
        }
        excluded_prefixes: Iterable[str] = []

        result = _build_military_job_labels(military_prefixes, military_roles, excluded_prefixes)

        assert "soldiers" in result
        assert result["soldiers"]["males"] == "جنود"

    def test_includes_allowed_prefixes(self) -> None:
        military_prefixes: GenderedLabelMap = {
            "army": {"males": "جيش", "females": "جيش"},
        }
        military_roles: GenderedLabelMap = {}
        excluded_prefixes: Iterable[str] = []

        result = _build_military_job_labels(military_prefixes, military_roles, excluded_prefixes)

        assert "army" in result

    def test_excludes_specified_prefixes(self) -> None:
        military_prefixes: GenderedLabelMap = {
            "special": {"males": "خاص", "females": "خاصة"},
        }
        military_roles: GenderedLabelMap = {}
        excluded_prefixes = ["special"]

        result = _build_military_job_labels(military_prefixes, military_roles, excluded_prefixes)

        assert "special" not in result

    def test_creates_prefix_role_combinations(self) -> None:
        military_prefixes: GenderedLabelMap = {
            "army": {"males": "جيش", "females": "جيش"},
        }
        military_roles: GenderedLabelMap = {
            "soldiers": {"males": "جنود", "females": "جنوديات"},
        }
        excluded_prefixes: Iterable[str] = []

        result = _build_military_job_labels(military_prefixes, military_roles, excluded_prefixes)

        assert "army soldiers" in result
        assert "جنود" in result["army soldiers"]["males"]
        assert "جيش" in result["army soldiers"]["males"]

    def test_excluded_prefixes_still_used_in_combinations(self) -> None:
        military_prefixes: GenderedLabelMap = {
            "navy": {"males": "بحري", "females": "بحرية"},
        }
        military_roles: GenderedLabelMap = {
            "soldiers": {"males": "جنود", "females": "جنوديات"},
        }
        excluded_prefixes = ["navy"]

        result = _build_military_job_labels(military_prefixes, military_roles, excluded_prefixes)

        # Navy should be excluded as standalone but used in combinations
        assert "navy" not in result
        assert "navy soldiers" in result



"""Utilities for managing gendered Arabic labels used across job modules.

This module replaces hand-written dictionary concatenation with typed helper
functions.  Each helper keeps the original Arabic content intact while
documenting the logic used to combine masculine and feminine labels.
"""

from __future__ import annotations

from typing import Dict, Mapping, MutableMapping, TypedDict


class GenderedLabel(TypedDict):
    """Represent an Arabic label split into masculine and feminine forms."""

    males: str
    females: str


GenderedLabelMap = Dict[str, GenderedLabel]


def combine_gender_labels(first_label: str, second_label: str) -> str:
    label = f"{first_label} {second_label}" if first_label and second_label else ""
    return label


def join_terms(*terms: str) -> str:
    """
    Join non-empty, whitespace-trimmed terms with a single space.

    Parameters:
        *terms (str): Terms to concatenate; each term is stripped of surrounding whitespace and empty or whitespace-only terms are ignored.

    Returns:
        str: Concatenated string of remaining terms separated by a single space.
    """

    filtered_terms = [term.strip() for term in terms if term and term.strip()]
    return " ".join(filtered_terms)


def copy_gendered_map(source: Mapping[str, GenderedLabel]) -> GenderedLabelMap:
    """
    Create a deep copy of a mapping from keys to gendered Arabic labels.

    Parameters:
        source (Mapping[str, GenderedLabel]): Mapping whose entries will be copied.

    Returns:
        GenderedLabelMap: A new mapping where each value is a newly constructed dict
        with the same "males" and "females" strings as the corresponding entry in
        `source`.
    """

    return {key: {"males": value["males"], "females": value["females"]} for key, value in source.items()}


def merge_gendered_maps(
    target: MutableMapping[str, GenderedLabel],
    source: Mapping[str, GenderedLabel],
) -> None:
    """Update ``target`` with copies from ``source`` to avoid shared state."""

    for key, value in source.items():
        target[key] = {"males": value["males"], "females": value["females"]}


def ensure_gendered_label(
    target: MutableMapping[str, GenderedLabel],
    key: str,
    value: GenderedLabel,
) -> None:
    """Insert ``value`` into ``target`` if ``key`` is not present."""

    if key not in target:
        target[key] = {"males": value["males"], "females": value["females"]}


def combine_gendered_labels(
    base_labels: GenderedLabel,
    suffix_labels: GenderedLabel,
    *,
    require_base_womens: bool = False,
) -> GenderedLabel:
    """Merge two :class:`GenderedLabel` mappings into a new mapping.

    Args:
        base_labels: The primary role labels.
        suffix_labels: The modifiers appended to the base labels.  These may be
            prefixes (e.g. ``"متزلجو"``) or suffixes (e.g. ``"أولمبيون"``).
        require_base_womens: When ``True`` the feminine label is emitted only if
            the base feminine label is available.  This mirrors the legacy
            behaviour used for some religious titles where the feminine form
            should remain empty unless explicitly defined for the base role.

    Returns:
        A new mapping containing concatenated masculine and feminine labels.

    Notes:
        ``combine_gendered_labels`` centralises the join logic that previously
        appeared across multiple modules.  Using the helper ensures consistent
        trimming behaviour and keeps future changes confined to a single
        implementation.
    """

    mens_label = join_terms(base_labels["males"], suffix_labels["males"])
    womens_label = ""
    if not require_base_womens or base_labels["females"]:
        womens_label = join_terms(base_labels["females"], suffix_labels["females"])
    return {"males": mens_label, "females": womens_label}


__all__ = [
    "GenderedLabel",
    "GenderedLabelMap",
    "combine_gendered_labels",
    "copy_gendered_map",
    "ensure_gendered_label",
    "join_terms",
    "merge_gendered_maps",
]



"""
Resolver for sports-related category labels with gender-specific logic.
This module provides functions to translate categories combining sports roles,
nationalities, and genders into idiomatic Arabic.
"""

import functools
import logging

from ...translations import SPORT_KEY_RECORDS_BASE, All_Nat
from ...translations_formats import MultiDataFormatterBaseV2, format_multi_data_v2
from .utils import fix_keys

logger = logging.getLogger(__name__)


def generate_sports_data_dict() -> dict[str, dict[str, str]]:
    """
    Build a mapping of sport keys to their Arabic role labels.

    Includes entries from SPORT_KEY_RECORDS_BASE where the record provides a non-empty "jobs" value (stored under the "sport_ar" key) and adds explicit Arabic mappings for several sports that are not present or need overrides.

    Returns:
        dict[str, dict[str, str]]: A dictionary where each English sport key maps to a dictionary with the key "sport_ar" containing the Arabic translation.
    """
    sports_data_new = {
        sport: {"sport_ar": record.get("jobs", "")}
        for sport, record in SPORT_KEY_RECORDS_BASE.items()
        if record.get("jobs", "")
    }

    sports_data_new.update(
        {
            "softball": {"sport_ar": "كرة لينة"},
            "futsal": {"sport_ar": "كرة صالات"},
            "badminton": {"sport_ar": "تنس ريشة"},
            "australian rules football": {"sport_ar": "كرة قدم أسترالية"},
            "american-football": {"sport_ar": "كرة قدم أمريكية"},
            "canadian-football": {"sport_ar": "كرة قدم كندية"},
        }
    )

    return sports_data_new


@functools.lru_cache(maxsize=1)
def _make_sport_formatted_data() -> dict[str, str]:
    formatted_data = {
        # _build_players_data - footballers without nats
        "male footballers": "لاعبو كرة قدم",
        "female footballers": "لاعبات كرة قدم",
        "footballers": "لاعبو ولاعبات كرة قدم",
        # _build_players_data - footballers with nats
        "{en_nat} male footballers": "لاعبو كرة قدم {males}",
        "{en_nat} female footballers": "لاعبات كرة قدم {females}",
        "{en_nat} footballers": "لاعبو ولاعبات كرة قدم {males}",
        # _build_players_data - sports without nats
        "female {en_sport} players": "لاعبات {sport_ar}",
        # _build_players_data - sports with nats
        "{en_nat} female {en_sport} players": "لاعبات {sport_ar} {females}",
        # _build_players_data - sports without nats
        "male {en_sport} players": "لاعبو {sport_ar}",
        "{en_sport} players": "لاعبو ولاعبات {sport_ar}",
        # _build_players_data - sports with nats
        "{en_nat} male {en_sport} players": "لاعبو {sport_ar} {males}",
        "{en_nat} {en_sport} players": "لاعبو ولاعبات {sport_ar} {males}",
    }

    players_of_data = {
        "australian rules football": "كرة قدم أسترالية",
        "american-football": "كرة قدم أمريكية",
    }
    for sport, ar in players_of_data.items():
        formatted_data.update(
            {
                f"female players of {sport}": f"لاعبات {ar}",
                f"{{en_nat}} female players of {sport}": f"لاعبات {ar} {{females}}",
                f"male players of {sport}": f"لاعبو {ar}",
                f"{{en_nat}} male players of {sport}": f"لاعبو {ar} {{males}}",
                f"players of {sport}": f"لاعبو ولاعبات {ar}",
                f"{{en_nat}} players of {sport}": f"لاعبو ولاعبات {ar} {{males}}",
            }
        )

    return formatted_data


@functools.lru_cache(maxsize=1)
def _sport_bot() -> MultiDataFormatterBaseV2:
    sports_data_new = generate_sports_data_dict()

    nats_data = {
        x: {
            "males": v["males"],
            "females": v["females"],
        }
        for x, v in All_Nat.items()
        if v.get("males")
    }
    formatted_data = _make_sport_formatted_data()

    return format_multi_data_v2(
        formatted_data=formatted_data,
        data_list=nats_data,
        key_placeholder="{en_nat}",
        data_list2=sports_data_new,
        key2_placeholder="{en_sport}",
        text_after="",
        text_before="the ",
        search_first_part=True,
        use_other_formatted_data=True,
    )


@functools.lru_cache(maxsize=10000)
def genders_sports_resolver(category: str) -> str:
    normalized_category = fix_keys(category)
    logger.debug(f"<<yellow>> start {normalized_category=}")

    sport_bot = _sport_bot()
    result = sport_bot.search_all_other_first(normalized_category)
    result = sport_bot.prepend_arabic_category_prefix(category, result)

    logger.info(f"<<yellow>> end {category=}, {result=}")

    return result


__all__ = [
    "genders_sports_resolver",
]



"""
Unit tests for the get_and_label function in labels_country module.
"""

from __future__ import annotations

import pytest

from ArWikiCats.translations.funcs import get_and_label


@pytest.mark.unit
def test_get_and_label_returns_joined_entities(monkeypatch: pytest.MonkeyPatch) -> None:
    """Test that get_and_label joins two entities with 'and' correctly."""
    monkeypatch.setattr(
        "ArWikiCats.translations.funcs.get_from_new_p17_final",
        lambda name, _: {"artist": "فنان", "painter": "رسام"}.get(name, ""),
    )

    result = get_and_label("Artist and Painter")
    assert result == "فنان ورسام"



#!/usr/bin/python3
"""
This module provides functions for processing and generating labels for country names based on separators.

Note: This module uses a callback pattern for `fetch_country_term_label` to avoid circular imports.
The callback is set at runtime via `set_term_label_resolver()`.
"""

import functools
import logging
import re
from typing import Callable, Optional, Tuple

from ...format_bots.relation_mapping import translation_category_relations
from ...new_resolvers import all_new_resolvers
from ...new_resolvers.bys_new import resolve_by_labels
from ...sub_new_resolvers import parties_resolver, team_work
from ...sub_new_resolvers.peoples_resolver import work_peoples
from ...translations.funcs import get_and_label, get_from_pf_keys2
from ..legacy_utils import fix_minor, split_text_by_separator
from ..make_bots import add_to_Films_O_TT, check_key_new_players, get_KAKO
from . import bys, with_years_bot
from .bot_2018 import get_pop_All_18

logger = logging.getLogger(__name__)

# Type alias for the term label resolver callback
TermLabelResolver = Callable[[str, str, str], str]

# Module-level callback holder - set via set_term_label_resolver()
_term_label_resolver: Optional[TermLabelResolver] = None


def set_term_label_resolver(resolver: TermLabelResolver) -> None:
    """
    Set the term label resolver callback.

    This is used to break the circular dependency. The resolver
    (typically fetch_country_term_label) is injected at runtime
    after all modules are loaded.

    Parameters:
        resolver: A callable that takes (term_lower: str, separator: str, lab_type: str)
                  and returns an Arabic label string.
    """
    global _term_label_resolver
    _term_label_resolver = resolver


def _get_term_label(term: str, separator: str = "", lab_type: str = "") -> str:
    """
    Get label from the term label resolver if one is set.

    Parameters:
        term: The term to resolve.
        separator: The separator context.
        lab_type: The label type (e.g., "type_label").

    Returns:
        The resolved label from the term label resolver, or empty string if not set.
    """
    if _term_label_resolver is not None:
        return _term_label_resolver(term, separator, lab_type)
    return ""


def get_table_with_in(cone_1: str, separator: str) -> str:
    """
    Map a composite key of cone_1 and separator to a predefined Arabic label.

    Parameters:
        cone_1 (str): Left-hand token to combine with the separator (e.g., "sport").
        separator (str): Separator token (e.g., "in").

    Returns:
        str: The mapped Arabic label for "{cone_1} {separator}", or an empty string if no mapping exists.
    """
    table_with_in = {
        "sport in": "الرياضة في",
    }
    con_1_in = f"{cone_1.strip()} {separator.strip()}"
    part_1_label = table_with_in.get(con_1_in, "")
    logger.debug(f"<<<< {con_1_in=}, {part_1_label=}")

    return part_1_label


@functools.lru_cache(maxsize=10000)
def resolve_part_1_label(country2, separator="", lab_type="type_label", with_years=False):
    """
    Resolve the Arabic label for the first part of a compound title given its English token and separator.

    Parameters:
        separator (str): The textual separator between parts (e.g., "in", "from", "to"); used to influence resolution for some tokens.
        country2 (str): The first English part to resolve (case-insensitive).

    Returns:
        str: The resolved Arabic label for `country2`, or an empty string if no mapping is found.

    Notes:
        - Special-case: when `country2` equals "women" and `separator` is "from", returns "نساء".

    Example:
        {"separator": " in ", "country2": "cultural depictions of competitors", "output": "تصوير ثقافي عن منافسون"},
    """
    if country2 == "women" and separator.strip() == "from":
        part_1_label = "نساء"
        logger.debug(f">> >> >> Make {country2=}.")
        return part_1_label

    country2_label = (
        ""
        or all_new_resolvers(country2)
        or get_pop_All_18(country2)
        or work_peoples(country2)
        or parties_resolver.get_parties_lab(country2)
        or team_work.resolve_clubs_teams_leagues(country2)
        or get_table_with_in(country2, separator)
        or get_from_pf_keys2(country2)
        or get_KAKO(country2)
        or _get_term_label(country2, "", lab_type=lab_type)
        or bys.get_by_label(country2)
        or get_and_label(country2)
        or (with_years_bot.Try_With_Years(country2) if with_years else "")
        or ""
    )

    return country2_label


def _resolve_war(resolved_label: str, part_2_normalized: str, part_1_normalized: str) -> str:
    """
    Return a corrected Arabic war label when appropriate.

    If `part_2_normalized` is a four-digit year and `part_1_normalized` equals `"war of"`,
    converts `"الحرب في {year}"` to `"حرب {year}"`. Otherwise returns `resolved_label` unchanged.

    Parameters:
        resolved_label (str): Current resolved Arabic label.
        part_2_normalized (str): Normalized second part (often a year).
        part_1_normalized (str): Normalized first part.

    Returns:
        str: The possibly modified label.
    """
    maren = re.match(r"\d\d\d\d", part_2_normalized)
    if maren:
        if part_1_normalized == "war of" and resolved_label == f"الحرب في {part_2_normalized}":
            resolved_label = f"حرب {part_2_normalized}"
            logger.info(f'<<lightpurple>> >>>> change cnt_la to "{resolved_label}".')

    return resolved_label


def make_cnt_lab(
    country: str,
    part_2_label: str,
    part_1_label: str,
    part_1_normalized: str,
    part_2_normalized: str,
    ar_separator: str,
) -> str:
    """
    Builds a final Arabic label by combining part_1_label and part_2_label and applying contextual adjustments and normalizations.

    Parameters:
        country (str): Original country string used to record mappings when applicable.
        part_2_label (str): Resolved label for the second part (right-hand side).
        part_1_label (str): Resolved label for the first part (left-hand side).
        part_1_normalized (str): Normalized form of the first part used to decide special formatting rules.
        part_2_normalized (str): Normalized form of the second part used to decide recording and war-related rewrites.
        ar_separator (str): Arabic separator to join the two parts (e.g., " في ", " إلى ", " لدى ").

    Description:
        - Concatenates part_1_label, ar_separator, and part_2_label to form an initial label.
        - For entries flagged as "new players" (checked via part_1_normalized), conditionally inserts "من " before part_2_label when it starts with "أصل ", otherwise appends " في" if not already present; and records the computed label for the country unless the part_2_normalized exists in the main resolver.
        - Applies predefined format mappings for specific part_1_normalized keys (e.g., politics/military patterns).
        - Collapses duplicated " في في " sequences and normalizes whitespace.
        - Calls internal war-resolution logic to rewrite phrases like "الحرب في {year}" to "حرب {year}" when applicable.
        - Removes a trailing " في " if present.

    Returns:
        str: The final, normalized Arabic label.
    """
    country2 = country.lower().strip()

    resolved_label = part_1_label + ar_separator + part_2_label
    in_players = check_key_new_players(part_1_normalized.lower())

    if in_players:
        if in_players:
            if part_2_label.startswith("أصل "):
                logger.info(f">>>>>> Add من to {part_1_normalized=} part_1_normalized in New_players:")
                resolved_label = f"{part_1_label}{ar_separator}من {part_2_label}"
            else:
                logger.info(f">>>>>> Add في to {part_1_normalized=} part_1_normalized in New_players:")
                if not resolved_label.strip().endswith(" في"):
                    resolved_label += " في "

        if not resolve_by_labels(part_2_normalized):
            # Films_O_TT[country2] = resolved_label
            add_to_Films_O_TT(country2, resolved_label)
        else:
            logger.info("<<lightblue>>>>>> part_2_normalized in By table main")

    if part_2_label:
        label_format_mappings = {
            "politics of {}": "سياسة {}",
            "military installations of": "منشآت {} العسكرية",
        }
        if part_1_normalized in label_format_mappings:
            logger.info(
                f'<<lightblue>>>>>> part_1_normalized in label_format_mappings "{label_format_mappings[part_1_normalized]}":'
            )
            resolved_label = label_format_mappings[part_1_normalized].format(part_2_label)

    # NOTE: important to fix bug like: [sport in ottoman] = "الرياضة في في الدولة العثمانية" !

    resolved_label = resolved_label.replace(" في في ", " في ")

    logger.info(f'<<lightpurple>> >>>> country 2_tit "{country2}": label: {resolved_label}')

    resolved_label = " ".join(resolved_label.strip().split())
    resolved_label = _resolve_war(resolved_label, part_2_normalized, part_1_normalized)

    resolved_label = resolved_label.removesuffix(" في ")

    return resolved_label


def separator_arabic_resolve(separator: str) -> str:
    """
    Map an English separator token to its Arabic equivalent, including surrounding spaces when appropriate.

    Parameters:
        separator (str): English separator token to map (e.g., "to", "on", "about", "based in").

    Returns:
        str: Arabic separator string (defaults to a single space if no mapping exists).
    """
    ar_separator = " "
    separator = separator.strip()

    if separator == "to":
        ar_separator = " إلى "
    elif separator == "on":
        ar_separator = " على "
    elif separator == "about":
        ar_separator = " عن "
    elif separator in translation_category_relations and separator != "by":
        ar_separator = f" {translation_category_relations[separator]} "
    elif separator == "based in":
        ar_separator = " مقرها في "

    return ar_separator


def make_parts_labels(part_1, part_2, separator, with_years) -> Tuple[str, str]:
    """
    Resolve Arabic labels for two text parts and return them as a pair.

    Attempts to derive each part's Arabic label from several resolvers and fallbacks; if either label cannot be resolved, returns two empty strings. When resolved, normalizes inputs and, depending on the English separator or trailing tokens, appends the Arabic preposition " في" to the first label for "in" contexts or prepends "من " to the second label for "from" contexts.

    Parameters:
        part_1 (str): The first text fragment to resolve (e.g., a role or type).
        part_2 (str): The second text fragment to resolve (e.g., a place or entity).
        separator (str): The English separator token between parts (affects Arabic preposition handling).
        with_years (bool): If true, allow an additional "with years" resolver when resolving part_2.

    Returns:
        Tuple[str, str]: (part_1_label, part_2_label) with resolved Arabic labels, or ("", "") if either could not be resolved.
    """
    part_2_label = resolve_part_1_label(part_2, with_years=with_years)

    if part_2_label == "" and "-" in part_2:
        part_2_label = resolve_part_1_label(part_2.replace("-", " "), with_years=with_years)

    part_1_label = resolve_part_1_label(part_1, separator)

    if part_1_label == "" and "-" in part_1:
        part_1_label = resolve_part_1_label(part_1.replace("-", " "), separator)

    if part_2_label == "" or part_1_label == "":
        logger.info(f">>>> XX--== <<lightgreen>> {part_1=}, {part_1_label=}, {part_2=}, {part_2_label=}")
        return "", ""

    part_1_normalized = part_1.strip().lower()
    part_2_normalized = part_2.strip().lower()

    if (separator.strip() == "in" or part_1_normalized.endswith(" in")) and (not part_1_normalized.endswith(" في")):
        logger.debug(f">>>> Add في to {part_1_label=}")
        part_1_label = f"{part_1_label} في"

    elif (separator.strip() == "from" or part_2_normalized.endswith(" from")) and (not part_2_label.endswith(" من")):
        logger.debug(f">>>> Add من to {part_2_label=}")
        part_2_label = f"من {part_2_label}"

    return part_1_label, part_2_label


def get_separator(country: str) -> str:
    """Get the separator from a country string.

    Args:
        country: The country string to check for separators

    Returns:
        The found separator or an empty string if none found
    """
    title_separators = [
        "based in",
        "in",
        "by",
        "about",
        "to",
        "of",
        "-of ",  # special case
        "from",
        "at",
        "on",
    ]

    normalized_country = country.lower().strip()

    for sep in title_separators:
        separator = f" {sep} " if sep != "-of " else sep
        if separator in normalized_country:
            return separator

    return ""


def country_2_title_work(country: str, with_years: bool = True) -> str:
    separator = get_separator(country)

    if not separator:
        logger.info(f">>>> <<red>> {separator=}")
        return ""

    part_1, part_2 = split_text_by_separator(separator, country)

    logger.info(f"2060 {part_1=}, {part_2=}, {separator=}")

    part_1_label, part_2_label = make_parts_labels(part_1, part_2, separator, with_years)

    if part_2_label == "" or part_1_label == "":
        logger.info(f">>>> XX--== <<lightgreen>> {part_1=}, {part_1_label=}, {part_2=}, {part_2_label=}")
        return ""

    part_1_normalized = part_1.strip().lower()
    part_2_normalized = part_2.strip().lower()

    logger.info(
        f">>>> XX--== <<lightgreen>> {part_1_normalized=}, {part_1_label=}, {part_2_normalized=}, {part_2_label=}"
    )

    if separator.strip() == "to" and (
        part_1_label.startswith("سفراء ") or part_1_normalized.strip() == "ambassadors of"
    ):
        ar_separator = " لدى "
    else:
        ar_separator = separator_arabic_resolve(separator)

    resolved_label = make_cnt_lab(
        country=country,
        part_2_label=part_2_label,
        part_1_label=part_1_label,
        part_1_normalized=part_1_normalized,
        part_2_normalized=part_2_normalized,
        ar_separator=ar_separator,
    )

    resolved_label = fix_minor(resolved_label, separator)

    return resolved_label


__all__ = [
    "country_2_title_work",
    "set_term_label_resolver",
]



# tests/utils/dump_runner.py
from __future__ import annotations

from collections.abc import Iterable

import pytest

ToTest = Iterable[tuple[str, dict[str, str]]]
ToTestCallback = Iterable[tuple[str, dict[str, str], callable]]


def _run_dump_case(name: str, data: dict[str, str], callback: callable, run_same=False, just_dump=False) -> None:
    """
    Common dump test logic shared across many test files.
    """
    from load_one_data import (
        dump_diff,
        dump_same_and_not_same,
        one_dump_test,
    )

    expected, diff_result = one_dump_test(data, callback)
    dump_diff(diff_result, name)

    if run_same:
        dump_same_and_not_same(data, diff_result, name, just_dump=just_dump)

    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"


def make_dump_test_name_data(to_test: ToTest, callback, run_same=False, just_dump=False):
    """
    Create a parametrized pytest test function.
    """

    @pytest.mark.parametrize("name,data", list(to_test))
    @pytest.mark.dump
    def test_dump_all(name: str, data: dict[str, str]) -> None:
        _run_dump_case(name, data, callback, run_same=run_same, just_dump=just_dump)

    return test_dump_all


def make_dump_test_name_data_callback(to_test: ToTestCallback, run_same=False, just_dump=False):
    """
    Create a parametrized pytest test function.
    """

    @pytest.mark.parametrize("name,data,callback", list(to_test))
    @pytest.mark.dump
    def test_dump_all(name: str, data: dict[str, str], callback) -> None:
        _run_dump_case(name, data, callback, run_same=run_same, just_dump=just_dump)

    return test_dump_all



#
import pytest

from ArWikiCats import resolve_label_ar
from utils.dump_runner import make_dump_test_name_data

data0 = {}

data_1 = {
    "1690s missing person cases": "قضايا أشخاص مفقودين في عقد 1690",
    "1620s missing person cases": "قضايا أشخاص مفقودين في عقد 1620",
    "1580s missing person cases": "قضايا أشخاص مفقودين في عقد 1580",
    "1540s missing person cases": "قضايا أشخاص مفقودين في عقد 1540",
    "1550s missing person cases": "قضايا أشخاص مفقودين في عقد 1550",
    "1510s missing person cases": "قضايا أشخاص مفقودين في عقد 1510",
    "1520s missing person cases": "قضايا أشخاص مفقودين في عقد 1520",
    "1500s missing person cases": "قضايا أشخاص مفقودين في عقد 1500",
    "1590s missing person cases": "قضايا أشخاص مفقودين في عقد 1590",
    "1610s missing person cases": "قضايا أشخاص مفقودين في عقد 1610",
    "1630s missing person cases": "قضايا أشخاص مفقودين في عقد 1630",
    "1670s missing person cases": "قضايا أشخاص مفقودين في عقد 1670",
    "1570s missing person cases": "قضايا أشخاص مفقودين في عقد 1570",
    "1680s missing person cases": "قضايا أشخاص مفقودين في عقد 1680",
    "1460s missing person cases": "قضايا أشخاص مفقودين في عقد 1460",
    "1410s missing person cases": "قضايا أشخاص مفقودين في عقد 1410",
    "Film people from Northern Ireland": "أعلام أفلام من أيرلندا الشمالية",
    "20th-century architects from Northern Ireland": "معماريون من أيرلندا الشمالية في القرن 20",
    "20th-century engineers from Northern Ireland": "مهندسون من أيرلندا الشمالية في القرن 20",
    "20th-century musicians by instrument from Northern Ireland": "موسيقيون في القرن 20 حسب الآلة من أيرلندا الشمالية",
    "21st-century engineers from Northern Ireland": "مهندسون من أيرلندا الشمالية في القرن 21",
    "21st-century male artists from Northern Ireland": "فنانون ذكور من أيرلندا الشمالية في القرن 21",
    "21st-century medical doctors from Northern Ireland": "أطباء من أيرلندا الشمالية في القرن 21",
    "21st-century musicians by instrument from Northern Ireland": "موسيقيون في القرن 21 حسب الآلة من أيرلندا الشمالية",
    "21st-century painters from Northern Ireland": "رسامون من أيرلندا الشمالية في القرن 21",
    "21st-century women educators from Northern Ireland": "معلمات من أيرلندا الشمالية في القرن 21",
    "21st-century women medical doctors from Northern Ireland": "طبيبات من أيرلندا الشمالية في القرن 21",
    "Accountants from Northern Ireland": "محاسبون من أيرلندا الشمالية",
    "Anarchists from Northern Ireland": "لاسلطويون من أيرلندا الشمالية",
    "Archers from Northern Ireland": "نبالون من أيرلندا الشمالية",
    "Art historians from Northern Ireland": "مؤرخو الفن من أيرلندا الشمالية",
    "Audio engineers from Northern Ireland": "مهندسو صوت من أيرلندا الشمالية",
    "Badminton players from Northern Ireland": "لاعبو تنس ريشة من أيرلندا الشمالية",
    "Basketball players from Northern Ireland": "لاعبو كرة سلة من أيرلندا الشمالية",
    "Bassoonists from Northern Ireland": "عازفو باسون من أيرلندا الشمالية",
    "Canoeists from Northern Ireland": "متسابقو قوارب الكانوي من أيرلندا الشمالية",
    "Cardiologists from Northern Ireland": "أطباء قلب من أيرلندا الشمالية",
    "Choreographers from Northern Ireland": "مصممو رقص من أيرلندا الشمالية",
    "Classical scholars from Northern Ireland": "باحثون كلاسيكيون من أيرلندا الشمالية",
    "Comedy writers from Northern Ireland": "كتاب كوميديا من أيرلندا الشمالية",
    "Documentary filmmakers from Northern Ireland": "صانعو أفلام وثائقية من أيرلندا الشمالية",
    "Editorial cartoonists from Northern Ireland": "محررون كارتونيون من أيرلندا الشمالية",
    "Editors from Northern Ireland": "محررون من أيرلندا الشمالية",
    "Electronic music groups from Northern Ireland": "فرق موسيقى إلكترونية من أيرلندا الشمالية",
    "Epidemiologists from Northern Ireland": "علماء وبائيات من أيرلندا الشمالية",
    "Female archers from Northern Ireland": "نبالات من أيرلندا الشمالية",
    "Female badminton players from Northern Ireland": "لاعبات تنس ريشة من أيرلندا الشمالية",
    "Female equestrians from Northern Ireland": "فارسات خيول من أيرلندا الشمالية",
    "Female golfers from Northern Ireland": "لاعبات غولف من أيرلندا الشمالية",
    "Female judoka from Northern Ireland": "لاعبات جودو من أيرلندا الشمالية",
    "Female martial artists from Northern Ireland": "ممارسات فنون قتالية من أيرلندا الشمالية",
    "Female sport shooters from Northern Ireland": "لاعبات رماية من أيرلندا الشمالية",
    "Female swimmers from Northern Ireland": "سباحات من أيرلندا الشمالية",
    "Feminists from Northern Ireland": "نسويون من أيرلندا الشمالية",
    "Fiddlers from Northern Ireland": "عازفو كمان من أيرلندا الشمالية",
    "Film critics from Northern Ireland": "نقاد أفلام من أيرلندا الشمالية",
    "Film editors from Northern Ireland": "محررو أفلام من أيرلندا الشمالية",
    "Films scored by composers from Northern Ireland": "أفلام سجلها ملحنون من أيرلندا الشمالية",
    "Folk music groups from Northern Ireland": "فرق موسيقى تقليدية من أيرلندا الشمالية",
    "Gaelic footballers from Northern Ireland by county": "لاعبو كرة قدم غيلية من أيرلندا الشمالية حسب المقاطعة",
    "Gaelic footballers from Northern Ireland": "لاعبو كرة قدم غيلية من أيرلندا الشمالية",
    "Gamblers from Northern Ireland": "مقامرون من أيرلندا الشمالية",
    "Geologists from Northern Ireland": "جيولوجيون من أيرلندا الشمالية",
    "Horticulturists from Northern Ireland": "بستانيون من أيرلندا الشمالية",
    "Humorists from Northern Ireland": "فكاهيون من أيرلندا الشمالية",
    "Installation artists from Northern Ireland": "فنانون تنصيبيون من أيرلندا الشمالية",
    "Jazz musicians from Northern Ireland": "موسيقيو جاز من أيرلندا الشمالية",
    "Legal scholars from Northern Ireland": "أساتذة قانون من أيرلندا الشمالية",
    "Lists of politicians from Northern Ireland": "قوائم سياسيون من أيرلندا الشمالية",
    "Local politicians in Northern Ireland": "سياسيون محليون في أيرلندا الشمالية",
    "Lyricists from Northern Ireland": "شعراء غنائيون من أيرلندا الشمالية",
    "Male chefs from Northern Ireland": "طباخون ذكور من أيرلندا الشمالية",
    "Male fiddlers from Northern Ireland": "عازفو كمان ذكور من أيرلندا الشمالية",
    "Male sailors (sport) from Northern Ireland": "بحارة رياضيون ذكور من أيرلندا الشمالية",
    "Male sport shooters from Northern Ireland": "لاعبو رماية ذكور من أيرلندا الشمالية",
    "Marxists from Northern Ireland": "ماركسيون من أيرلندا الشمالية",
    "Meteorologists from Northern Ireland": "خبراء أرصاد جوية من أيرلندا الشمالية",
    "Music critics from Northern Ireland": "نقاد موسيقى من أيرلندا الشمالية",
    "Music journalists from Northern Ireland": "صحفيون موسيقيون من أيرلندا الشمالية",
    "Music video directors from Northern Ireland": "مخرجو فيديوهات موسيقية من أيرلندا الشمالية",
    "Musicologists from Northern Ireland": "علماء موسيقى من أيرلندا الشمالية",
    "Nature writers from Northern Ireland": "كتاب طبيعة من أيرلندا الشمالية",
    "Netball players from Northern Ireland": "لاعبو كرة شبكة من أيرلندا الشمالية",
    "Neurologists from Northern Ireland": "أطباء أعصاب من أيرلندا الشمالية",
    "Neuroscientists from Northern Ireland": "علماء أعصاب من أيرلندا الشمالية",
    "Newspaper editors from Northern Ireland": "محررو صحف من أيرلندا الشمالية",
    "Newspaper people by newspaper in Northern Ireland": "شخصيات صحف حسب الصحيفة في أيرلندا الشمالية",
    "Newspaper people from Northern Ireland": "شخصيات صحف من أيرلندا الشمالية",
    "Nurses from Northern Ireland": "ممرضون من أيرلندا الشمالية",
    "Organists from Northern Ireland": "عازفو أورغن من أيرلندا الشمالية",
    "Ornithologists from Northern Ireland": "علماء طيور من أيرلندا الشمالية",
    "Poker players from Northern Ireland": "لاعبو بوكر من أيرلندا الشمالية",
    "Political scientists from Northern Ireland": "علماء سياسة من أيرلندا الشمالية",
    "Political writers from Northern Ireland": "كتاب سياسة من أيرلندا الشمالية",
    "Pop music groups from Northern Ireland": "فرق موسيقى بوب من أيرلندا الشمالية",
    "Radio actresses from Northern Ireland": "ممثلات راديو من أيرلندا الشمالية",
    "Radio producers from Northern Ireland": "منتجو راديو من أيرلندا الشمالية",
    "Radiologists from Northern Ireland": "أخصائيو أشعة من أيرلندا الشمالية",
    "Sailors (sport) from Northern Ireland": "بحارة رياضيون من أيرلندا الشمالية",
    "Satirists from Northern Ireland": "كتاب هجائيون من أيرلندا الشمالية",
    "Silent film actors from Northern Ireland": "ممثلو أفلام صامتة من أيرلندا الشمالية",
    "Sport shooters from Northern Ireland": "لاعبو رماية من أيرلندا الشمالية",
    "Sports broadcasters from Northern Ireland": "مذيعون رياضيون من أيرلندا الشمالية",
    "Sportspeople from Northern Ireland in doping cases": "رياضيون من أيرلندا الشمالية في قضايا منشطات",
    "Statisticians from Northern Ireland": "إحصائيون من أيرلندا الشمالية",
    "Surrealist artists from Northern Ireland": "فنانون سرياليون من أيرلندا الشمالية",
    "Table tennis players from Northern Ireland": "لاعبو كرة طاولة من أيرلندا الشمالية",
    "Television chefs from Northern Ireland": "طباخو تلفاز من أيرلندا الشمالية",
    "Television talk show hosts from Northern Ireland": "مقدمو برامج تلفزيونية حوارية من أيرلندا الشمالية",
    "Video artists from Northern Ireland": "فنانو فيديو من أيرلندا الشمالية",
    "Watercolourists from Northern Ireland": "رسامون مائيون من أيرلندا الشمالية",
    "Women academics from Northern Ireland": "أكاديميات من أيرلندا الشمالية",
    "Women boxers from Northern Ireland": "ملاكمات من أيرلندا الشمالية",
    "Women chefs from Northern Ireland": "طباخات من أيرلندا الشمالية",
    "Women choreographers from Northern Ireland": "مصممات رقص من أيرلندا الشمالية",
    "Women civil servants from Northern Ireland": "موظفات خدمة مدنية من أيرلندا الشمالية",
    "Women comedians from Northern Ireland": "كوميديات من أيرلندا الشمالية",
    "Women judges from Northern Ireland": "قاضيات من أيرلندا الشمالية",
    "Women mathematicians from Northern Ireland": "عالمات رياضيات من أيرلندا الشمالية",
    "Women mayors of places in Northern Ireland": "رئيسات بلديات في أيرلندا الشمالية",
    "Women members of the Parliament of Northern Ireland": "عضوات برلمان أيرلندا الشمالية",
    "Women photographers from Northern Ireland": "مصورات من أيرلندا الشمالية",
    "Women television personalities from Northern Ireland": "شخصيات تلفزيون من أيرلندا الشمالية",
    "Women travel writers from Northern Ireland": "كاتبات سفر من أيرلندا الشمالية",
}

data_2 = {
    "20th-century MEPs for Northern Ireland": "",
    "20th-century Presbyterian ministers from Northern Ireland": "",
    "21st-century MEPs for Northern Ireland": "",
    "21st-century Presbyterian ministers from Northern Ireland": "",
    "21st-century women MEPs for Northern Ireland": "",
    "Advocates general for Northern Ireland": "",
    "All-female bands from Northern Ireland": "",
    "Alliance Party of Northern Ireland politicians": "",
    "Anaesthetists from Northern Ireland": "",
    "Antiquarians from Northern Ireland": "",
    "Artistic directors from Northern Ireland": "",
    "Banjoists from Northern Ireland": "",
    "Barristers from Northern Ireland": "",
    "Blues musical groups from Northern Ireland": "",
    "Bouzouki players from Northern Ireland": "",
    "Boxing promoters from Northern Ireland": "",
    "Ceremonial officers in Northern Ireland": "",
    "Chief Medical Officers for Northern Ireland": "",
    "Children's entertainers from Northern Ireland": "",
    "Children's television presenters from Northern Ireland": "",
    "Civil servants in the Northern Ireland Office": "",
    "Cookbook writers from Northern Ireland": "",
    "Councillors in Northern Ireland": "",
    "Deputy lieutenants in Northern Ireland": "",
    "Drag queens from Northern Ireland": "",
    "Experimental musical groups from Northern Ireland": "",
    "Far-right politicians from Northern Ireland": "",
    "Film score composers from Northern Ireland": "",
    "Folk-song collectors from Northern Ireland": "",
    "Game show hosts from Northern Ireland": "",
    "Gay actors from Northern Ireland": "",
    "Gay men from Northern Ireland by occupation": "",
    "Gay musicians from Northern Ireland": "",
    "Gay writers from Northern Ireland": "",
    "General practitioners from Northern Ireland": "",
    "Green Party Northern Ireland politicians": "",
    "Hammered dulcimer players from Northern Ireland": "",
    "Harpists from Northern Ireland": "",
    "Heads of the Northern Ireland Civil Service": "",
    "Health professionals from Northern Ireland by county": "",
    "Health professionals from Northern Ireland": "",
    "High Court judges of Northern Ireland": "",
    "High shrievalties in Northern Ireland": "",
    "Hurlers from Northern Ireland by county": "",
    "Hurlers from Northern Ireland": "",
    "Hymnwriters from Northern Ireland": "",
    "Independent politicians in Northern Ireland": "",
    "Ladies' Gaelic footballers from Northern Ireland": "",
    "Leaders of the Alliance Party of Northern Ireland": "",
    "LGBTQ comedians from Northern Ireland": "",
    "LGBTQ dramatists and playwrights from Northern Ireland": "",
    "LGBTQ models from Northern Ireland": "",
    "Local government officers in Northern Ireland": "",
    "Lord chief justices of Northern Ireland": "",
    "Lord-lieutenants in Northern Ireland": "",
    "Lords Justice of Appeal of Northern Ireland": "",
    "Male harpists from Northern Ireland": "",
    "Male uilleann pipers from Northern Ireland": "",
    "Marxist writers from Northern Ireland": "",
    "Mass media people from Northern Ireland by county": "",
    "Media personalities from Northern Ireland": "",
    "Members of the Northern Ireland Civil Service": "",
    "Members of the Northern Ireland Constitutional Convention": "",
    "Members of the Northern Ireland Forum": "",
    "Members of the Privy Council of Northern Ireland": "",
    "Methodist ministers from Northern Ireland": "",
    "Musical duos from Northern Ireland": "",
    "Musical quartets from Northern Ireland": "",
    "Musical quintets from Northern Ireland": "",
    "Musical theatre composers from Northern Ireland": "",
    "Musical trios from Northern Ireland": "",
    "National Democratic Party (Northern Ireland) politicians": "",
    "Noise musical groups from Northern Ireland": "",
    "Northern Ireland Government ministers": "",
    "Northern Ireland justices of the peace": "",
    "Northern Ireland King's Counsel": "",
    "Northern Ireland Labour Party politicians": "",
    "Northern Ireland MLAs 1998–2003": "",
    "Northern Ireland prison officers": "",
    "Northern Ireland Unionist Party politicians": "",
    "Northern Ireland Women's Coalition politicians": "",
    "Oboists from Northern Ireland": "",
    "Ombudsmen in Northern Ireland": "",
    "Orchestras from Northern Ireland": "",
    "Palaeontologists from Northern Ireland": "",
    "Permanent under-secretaries of state for Northern Ireland": "",
    "Players of English billiards from Northern Ireland": "",
    "Pool players from Northern Ireland": "",
    "Presbyterian ministers from Northern Ireland": "",
    "Psychologists from Northern Ireland": "",
    "Punk rock musicians from Northern Ireland": "",
    "Radio DJs from Northern Ireland": "",
    "Roboticists from Northern Ireland": "",
    "Shakespearean actors from Northern Ireland": "",
    "Songs by songwriters from Northern Ireland": "",
    "Strength athletes from Northern Ireland": "",
    "TikTokers from Northern Ireland": "",
    "Tin whistle players from Northern Ireland": "",
    "Uilleann pipers from Northern Ireland": "",
    "Wikipedia categories named after actors from Northern Ireland": "",
    "Wikipedia categories named after musical groups from Northern Ireland": "",
    "Wikipedia categories named after musicians from Northern Ireland": "",
    "Women councillors in Northern Ireland": "",
    "Women harpists from Northern Ireland": "",
    "Women MEPs for Northern Ireland": "",
    "Women political office-holders in Northern Ireland": "",
}

to_test = [
    ("test_northern_ireland_1", data_1),
    # ("test_northern_ireland_2", data_2),
]


@pytest.mark.parametrize("category, expected", data_1.items(), ids=data_1.keys())
@pytest.mark.fast
def test_northern_ireland_1(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


test_dump_all = make_dump_test_name_data(to_test, resolve_label_ar, run_same=False)



"""
Package for resolving country names in category titles.
This package provides specialized resolvers for matching and translating
country names and related geographic entities (like US states) into Arabic.
"""

import functools
import logging

from . import (  # countries_names_double_v2,
    countries_names,
    countries_names_v2,
    geo_names_formats,
    medalists_resolvers,
    us_states,
)

logger = logging.getLogger(__name__)


@functools.lru_cache(maxsize=10000)
def main_countries_names_resolvers(normalized_category: str) -> str:
    """
    Orchestrates resolution of an Arabic label for geographic category strings using a prioritized sequence of resolvers.

    The input is normalized by stripping whitespace, converting to lowercase, and removing a leading "category:" before resolution.

    Parameters:
        normalized_category (str): Category string to resolve; it will be normalized as described above.

    Returns:
        str: Resolved Arabic label, or an empty string if no resolver produces a match.
    """
    normalized_category = normalized_category.strip().lower().replace("category:", "")
    logger.debug("--" * 20)
    logger.debug(f"<><><><><><> <<green>> {normalized_category=}")

    resolved_label = (
        # NOTE: order matters here
        # resolve_by_countries_names_v2 must be before resolve_by_countries_names, to avoid mis-resolving like:
        # incorrect:    [Category:Zimbabwe political leader] : "تصنيف:قادة زيمبابوي السياسيون",
        # correct:      [Category:Zimbabwe political leader] : "تصنيف:قادة سياسيون زيمبابويون",
        countries_names_v2.resolve_by_countries_names_v2(normalized_category)
        or countries_names.resolve_by_countries_names(normalized_category)
        or medalists_resolvers.resolve_countries_names_medalists(normalized_category)
        or us_states.resolve_us_states(normalized_category)
        or geo_names_formats.resolve_by_geo_names(normalized_category)
        # or countries_names_double_v2.resolve_countries_names_double(normalized_category)
        or ""
    )

    logger.info(f"<<yellow>> end {normalized_category=}, {resolved_label=}")
    return resolved_label


__all__ = [
    "main_countries_names_resolvers",
]



"""Make bots module for Arabic Wikipedia categories processing.

This package contains various utilities for processing and transforming
Wikipedia category names from English to Arabic.
"""

from .bot import Films_O_TT, add_to_Films_O_TT, players_new_keys
from .check_bot import check_key_new_players
from .reg_result import get_cats, get_reg_result
from .table1_bot import get_KAKO

__all__ = [
    "check_key_new_players",
    "get_KAKO",
    "Films_O_TT",
    "players_new_keys",
    "add_to_Films_O_TT",
    "get_cats",
    "get_reg_result",
]



#!/usr/bin/python3
"""
Module for handling year-and-from based category translations.

This module provides classes for formatting template-driven translation labels
that combine temporal patterns (years, decades, centuries) with "from" relation
patterns (e.g., "writers from Yemen", "people from Germany").

The module integrates with category_relation_mapping to resolve relation words
(prepositions like "from", "in", "by") into their Arabic equivalents.

Classes:
    FormatDataFrom: A dynamic wrapper for handling category transformations
        with customizable callbacks for key matching and searching.
    MultiDataFormatterYearAndFrom: Combines year-based and "from" relation
        category translations using the parent class helpers.

Example:
    >>> from ArWikiCats.translations_formats import MultiDataFormatterYearAndFrom, FormatDataFrom
    >>> country_bot = FormatDataFrom(
    ...     formatted_data={"{year1} {country1}": "{country1} في {year1}"},
    ...     key_placeholder="{country1}",
    ...     value_placeholder="{country1}",
    ...     search_callback=get_label_func,
    ...     match_key_callback=match_key_func,
    ... )
    >>> year_bot = FormatDataFrom(
    ...     formatted_data={},
    ...     key_placeholder="{year1}",
    ...     value_placeholder="{year1}",
    ...     search_callback=convert_time_to_arabic,
    ...     match_key_callback=match_time_en_first,
    ... )
    >>> bot = MultiDataFormatterYearAndFrom(country_bot, year_bot, other_key_first=True)
    >>> bot.create_label("14th-century writers from Yemen")
    'كتاب من اليمن في القرن 14'
"""

from ..DataModel import FormatDataFrom
from .model_multi_data_base import MultiDataFormatterBaseHelpers


class MultiDataFormatterYearAndFrom(MultiDataFormatterBaseHelpers):
    """
    Combines year-based and "from" relation category translations.

    This class orchestrates two FormatDataFrom instances (country_bot and year_bot)
    to normalize and translate category strings that contain both temporal patterns
    and "from" relation patterns.

    The class integrates with category_relation_mapping to resolve relation words
    (prepositions like "from", "in", "by") into their Arabic equivalents when
    building labels.

    Attributes:
        country_bot (FormatDataFrom): Handles the "from" relation part (e.g., "writers from Yemen").
        other_bot (FormatDataFrom): Handles the year/time part (e.g., "14th-century").
        search_first_part (bool): If True, search using only the first part (after country normalization).
        data_to_find (dict[str, str] | None): Optional direct lookup dictionary for category labels.
        other_key_first (bool): If True, process the year/other key before the country key.

    """

    def __init__(
        self,
        country_bot: FormatDataFrom,
        year_bot: FormatDataFrom,
        search_first_part: bool = False,
        data_to_find: dict[str, str] | None = None,
        other_key_first: bool = False,
    ) -> None:
        """Prepare helpers for matching and formatting template-driven labels.

        Args:
            country_bot: FormatDataFrom instance for handling "from" relation patterns.
            year_bot: FormatDataFrom instance for handling year/time patterns.
            search_first_part: If True, search using only the first part after normalization.
            data_to_find: Optional dictionary for direct category-to-label lookups.
            other_key_first: If True, process year/other key before country key.
        """
        self.search_first_part = search_first_part
        self.country_bot = country_bot
        self.other_bot = year_bot
        self.data_to_find = data_to_find
        self.other_key_first = other_key_first



"""Tests for build_jobs_players_list.py gendered player label utilities."""

from ArWikiCats.translations.data_builders.build_jobs_players_list import (
    GenderedLabelMap,
    _build_boxing_labels,
    _build_general_scope_labels,
    _build_jobs_player_variants,
    _build_skating_labels,
    _build_sports_job_variants,
    _build_team_sport_labels,
    _merge_maps,
)
from ArWikiCats.translations.data_builders.jobs_defs import combine_gender_labels


class TestBuildBoxingLabels:
    """Tests for _build_boxing_labels function."""

    def test_builds_boxer_labels(self) -> None:
        weights = {"heavyweight": "الوزن الثقيل", "flyweight": "الوزن الذبابي"}
        result = _build_boxing_labels(weights)

        assert "heavyweight boxers" in result
        assert result["heavyweight boxers"]["males"] == "ملاكمو الوزن الثقيل"
        assert result["heavyweight boxers"]["females"] == "ملاكمات الوزن الثقيل"

    def test_builds_world_champions(self) -> None:
        weights = {"welterweight": "الوزن الويلتر"}
        result = _build_boxing_labels(weights)

        assert "world welterweight boxing champions" in result
        assert result["world welterweight boxing champions"]["males"] == "أبطال العالم للملاكمة فئة الوزن الويلتر"

    def test_skips_empty_weights(self) -> None:
        weights = {"valid": "صحيح", "empty": ""}
        result = _build_boxing_labels(weights)

        assert "valid boxers" in result
        assert "empty boxers" not in result


class TestBuildSkatingLabels:
    """Tests for _build_skating_labels function."""

    def test_builds_skater_variants(self) -> None:
        labels = {
            "figure": {"males": "فني التزلج", "females": "فنية التزلج"},
            "speed": {"males": "سرعة التزلج", "females": "سرعة التزلج"},
        }
        result = _build_skating_labels(labels)

        assert "figure skaters" in result
        assert result["figure skaters"]["males"] == "متزلجو فني التزلج"
        assert result["figure skaters"]["females"] == "متزلجات فنية التزلج"

    def test_builds_skier_variants(self) -> None:
        labels = {
            "alpine": {"males": "تزلج جبالي", "females": "تزلج جبالي"},
        }
        result = _build_skating_labels(labels)

        assert "alpine skiers" in result
        assert result["alpine skiers"]["males"] == "متزحلقو تزلج جبالي"
        assert result["alpine skiers"]["females"] == "متزحلقات تزلج جبالي"


class TestBuildTeamSportLabels:
    """Tests for _build_team_sport_labels function."""

    def test_builds_team_labels(self) -> None:
        translations = {
            "football": "كرة القدم",
            "basketball": "كرة السلة",
        }
        result = _build_team_sport_labels(translations)

        assert result["football"]["males"] == "لاعبو كرة القدم"
        assert result["football"]["females"] == "لاعبات كرة القدم"

    def test_skips_empty_translations(self) -> None:
        translations = {"valid": "صحيح", "empty": ""}
        result = _build_team_sport_labels(translations)

        assert "valid" in result
        assert "empty" not in result


class TestBuildJobsPlayerVariants:
    """Tests for _build_jobs_player_variants function."""

    def test_adds_lowercase_key(self) -> None:
        players = {"Players": {"males": "لاعبو", "females": "لاعبات"}}
        result = _build_jobs_player_variants(players)

        assert "players" in result
        assert result["players"]["males"] == "لاعبو"

    def test_adds_olympic_variants(self) -> None:
        players = {"players": {"males": "لاعبو", "females": "لاعبات"}}
        result = _build_jobs_player_variants(players)

        assert "olympic players" in result
        assert result["olympic players"]["males"] == "لاعبو أولمبيون"
        assert result["olympic players"]["females"] == "لاعبات أولمبيات"

    def test_adds_international_variants(self) -> None:
        players = {"players": {"males": "لاعبو", "females": "لاعبات"}}
        result = _build_jobs_player_variants(players)

        assert "international players" in result
        assert result["international players"]["males"] == "لاعبو دوليون"
        assert result["international players"]["females"] == "لاعبات دوليات"

    def test_skips_empty_labels(self) -> None:
        players = {"empty": {"males": "", "females": ""}}
        result = _build_jobs_player_variants(players)

        assert "empty" not in result
        assert "olympic empty" not in result


class TestBuildGeneralScopeLabels:
    """Tests for _build_general_scope_labels function."""

    def test_combines_roles_and_scopes(self) -> None:
        roles = {"coach": {"males": "مدربو", "females": "مدربات"}}
        scopes = {"olympic": {"males": "أولمبيون", "females": "أولمبيات"}}
        result = _build_general_scope_labels(roles, scopes)

        assert "olympic coach" in result
        # Result should have combined labels
        assert result["olympic coach"]["males"] == combine_gender_labels("مدربو", "أولمبيون")

    def test_handles_empty_labels(self) -> None:
        roles = {"coach": {"males": "مدربو", "females": ""}}
        scopes = {"olympic": {"males": "", "females": "أولمبيات"}}
        result = _build_general_scope_labels(roles, scopes)

        assert "olympic coach" in result

class TestBuildSportsJobVariants:
    """Tests for _build_sports_job_variants function."""

    def test_builds_biography_variants(self) -> None:
        sport_jobs = {"football": "كرة القدم"}
        football_roles = {}
        result = _build_sports_job_variants(sport_jobs, football_roles)

        assert "football biography" in result
        assert result["football biography"]["males"] == "أعلام كرة القدم"

    def test_builds_coaches_variants(self) -> None:
        sport_jobs = {"basketball": "كرة السلة"}
        football_roles = {}
        result = _build_sports_job_variants(sport_jobs, football_roles)

        assert "basketball coaches" in result
        assert result["basketball coaches"]["males"] == "مدربو كرة السلة"

    def test_combines_with_football_roles(self) -> None:
        sport_jobs = {"football": "كرة القدم"}
        football_roles = {"goalkeeper": {"males": "حراس مرمى", "females": "حارسات مرمى"}}
        result = _build_sports_job_variants(sport_jobs, football_roles)

        # Should create composite entries
        assert "football goalkeeper" in result
        # Check that it combines labels properly
        assert (
            "goalkeeper" in result["football goalkeeper"]["males"]
            or "كرة القدم" in result["football goalkeeper"]["males"]
        )

    def test_skips_empty_jobs(self) -> None:
        sport_jobs = {"valid": "صحيح", "empty": ""}
        football_roles = {}
        result = _build_sports_job_variants(sport_jobs, football_roles)

        assert "valid biography" in result
        assert "empty biography" not in result


class TestMergeMaps:
    """Tests for _merge_maps function."""

    def test_merges_multiple_maps(self) -> None:
        map1: GenderedLabelMap = {"key1": {"males": "1", "females": "1f"}}
        map2: GenderedLabelMap = {"key2": {"males": "2", "females": "2f"}}
        map3: GenderedLabelMap = {"key3": {"males": "3", "females": "3f"}}

        result = _merge_maps(map1, map2, map3)

        assert "key1" in result
        assert "key2" in result
        assert "key3" in result

    def test_later_maps_override(self) -> None:
        map1: GenderedLabelMap = {"key": {"males": "first", "females": "first_f"}}
        map2: GenderedLabelMap = {"key": {"males": "second", "females": "second_f"}}

        result = _merge_maps(map1, map2)

        assert result["key"]["males"] == "second"

    def test_empty_maps(self) -> None:
        result = _merge_maps()
        assert result == {}



#!/usr/bin/python3
"""
Unit tests for data_with_time.py module.

This module provides tests for format_year_country_data and format_year_country_data_v2
factory functions which create MultiDataFormatterBaseYear and MultiDataFormatterBaseYearV2 instances.
"""

from ArWikiCats.translations_formats.data_with_time import (
    COUNTRY_PARAM,
    YEAR_PARAM,
    format_year_country_data,
    format_year_country_data_v2,
)
from ArWikiCats.translations_formats.DataModelMulti import (
    MultiDataFormatterBaseYear,
    MultiDataFormatterBaseYearV2,
)


class TestFormatYearCountryData:
    """Tests for format_year_country_data factory function."""

    def test_returns_multi_data_formatter_base_year(self):
        """Test that format_year_country_data returns a MultiDataFormatterBaseYear instance."""
        formatted_data = {"{year1} {country1} events": "{country1} أحداث في {year1}"}
        data_list = {"british": "بريطانية"}

        bot = format_year_country_data(
            formatted_data=formatted_data,
            data_list=data_list,
        )

        assert isinstance(bot, MultiDataFormatterBaseYear)

    def test_default_placeholders(self):
        """Test default placeholders are used correctly."""
        formatted_data = {"{year1} {country1} events": "{country1} أحداث في {year1}"}
        data_list = {"british": "بريطانية"}

        bot = format_year_country_data(
            formatted_data=formatted_data,
            data_list=data_list,
        )

        assert bot.country_bot.key_placeholder == COUNTRY_PARAM
        assert bot.country_bot.value_placeholder == COUNTRY_PARAM
        assert bot.other_bot.key_placeholder == YEAR_PARAM
        assert bot.other_bot.value_placeholder == YEAR_PARAM

    def test_custom_placeholders(self):
        """Test custom placeholders are used correctly."""
        formatted_data = {"{time} {nat} writers": "{nat_ar} كتاب في {time_ar}"}
        data_list = {"yemen": "يمنية"}

        bot = format_year_country_data(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{nat}",
            value_placeholder="{nat_ar}",
            key2_placeholder="{time}",
            value2_placeholder="{time_ar}",
        )

        assert bot.country_bot.key_placeholder == "{nat}"
        assert bot.country_bot.value_placeholder == "{nat_ar}"
        assert bot.other_bot.key_placeholder == "{time}"
        assert bot.other_bot.value_placeholder == "{time_ar}"

    def test_text_before_and_after(self):
        """Test text_before and text_after are passed to country_bot."""
        formatted_data = {"{year1} {country1} events": "{country1} أحداث في {year1}"}
        data_list = {"british": "بريطانية"}

        bot = format_year_country_data(
            formatted_data=formatted_data,
            data_list=data_list,
            text_before="the ",
            text_after=" !",
        )

        assert bot.country_bot.text_before == "the "
        assert bot.country_bot.text_after == " !"

    def test_data_to_find(self):
        """Test data_to_find is stored correctly."""
        formatted_data = {"{year1} {country1} events": "{country1} أحداث في {year1}"}
        data_list = {"british": "بريطانية"}
        data_to_find = {"direct lookup": "نتيجة مباشرة"}

        bot = format_year_country_data(
            formatted_data=formatted_data,
            data_list=data_list,
            data_to_find=data_to_find,
        )

        assert bot.data_to_find == data_to_find


class TestFormatYearCountryDataV2:
    """Tests for format_year_country_data_v2 factory function."""

    def test_returns_multi_data_formatter_base_year_v2(self):
        """Test that format_year_country_data_v2 returns a MultiDataFormatterBaseYearV2 instance."""
        formatted_data = {"{year1} {country1} writers": "{demonym} كتاب في {year1}"}
        data_list = {"yemen": {"demonym": "يمنيون"}}

        bot = format_year_country_data_v2(
            formatted_data=formatted_data,
            data_list=data_list,
        )

        assert isinstance(bot, MultiDataFormatterBaseYearV2)

    def test_default_placeholders(self):
        """Test default placeholders are used correctly."""
        formatted_data = {"{year1} {country1} writers": "{demonym} كتاب في {year1}"}
        data_list = {"yemen": {"demonym": "يمنيون"}}

        bot = format_year_country_data_v2(
            formatted_data=formatted_data,
            data_list=data_list,
        )

        assert bot.country_bot.key_placeholder == COUNTRY_PARAM
        assert bot.other_bot.key_placeholder == YEAR_PARAM
        assert bot.other_bot.value_placeholder == YEAR_PARAM

    def test_custom_placeholders(self):
        """Test custom placeholders are used correctly."""
        formatted_data = {"{time} {nat} writers": "{demonym} كتاب في {time_ar}"}
        data_list = {"yemen": {"demonym": "يمنيون"}}

        bot = format_year_country_data_v2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{nat}",
            key2_placeholder="{time}",
            value2_placeholder="{time_ar}",
        )

        assert bot.country_bot.key_placeholder == "{nat}"
        assert bot.other_bot.key_placeholder == "{time}"
        assert bot.other_bot.value_placeholder == "{time_ar}"

    def test_text_before_and_after(self):
        """Test text_before and text_after are passed to country_bot."""
        formatted_data = {"{year1} {country1} writers": "{demonym} كتاب في {year1}"}
        data_list = {"yemen": {"demonym": "يمنيون"}}

        bot = format_year_country_data_v2(
            formatted_data=formatted_data,
            data_list=data_list,
            text_before="the ",
            text_after=" !",
        )

        assert bot.country_bot.text_before == "the "
        assert bot.country_bot.text_after == " !"

    def test_data_to_find(self):
        """Test data_to_find is stored correctly."""
        formatted_data = {"{year1} {country1} writers": "{demonym} كتاب في {year1}"}
        data_list = {"yemen": {"demonym": "يمنيون"}}
        data_to_find = {"direct lookup": "نتيجة مباشرة"}

        bot = format_year_country_data_v2(
            formatted_data=formatted_data,
            data_list=data_list,
            data_to_find=data_to_find,
        )

        assert bot.data_to_find == data_to_find


class TestConstants:
    """Tests for module constants."""

    def test_year_param(self):
        """Test YEAR_PARAM constant."""
        assert YEAR_PARAM == "{year1}"

    def test_country_param(self):
        """Test COUNTRY_PARAM constant."""
        assert COUNTRY_PARAM == "{country1}"



"""Party label helpers."""

from __future__ import annotations

import logging

from ..translations import PARTIES
from ..translations_formats import FormatData

logger = logging.getLogger(__name__)

formatted_data = {
    "{party_key} candidates for member of parliament": "مرشحو {party_label} لعضوية البرلمان",
    "{party_key} candidates for member-of-parliament": "مرشحو {party_label} لعضوية البرلمان",
    "{party_key} candidates": "مرشحو {party_label}",
    "{party_key} leaders": "قادة {party_label}",
    "{party_key} politicians": "سياسيو {party_label}",
    "{party_key} members": "أعضاء {party_label}",
    "{party_key} state governors": "حكام ولايات من {party_label}",
}

_parties_bot = FormatData(
    formatted_data=formatted_data,
    data_list=PARTIES,
    key_placeholder="{party_key}",
    value_placeholder="{party_label}",
)


def get_parties_lab(party: str) -> str:
    """Return the Arabic label for ``party`` using known suffixes.

    Args:
        party: The party name to resolve.

    Returns:
        The resolved Arabic label or an empty string if the suffix is unknown.
    """

    normalized_party = party.strip()
    logger.debug(f" {party=}")

    # Try FormatData first
    label = _parties_bot.search(normalized_party)
    logger.info(f" {party=}, {label=}")

    return label


__all__ = ["get_parties_lab"]



#
import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats import resolve_label_ar

geography_data = {
    "Culture of Westchester County, New York": "ثقافة مقاطعة ويستتشستر (نيويورك)",
    "Economy of Westchester County, New York": "اقتصاد مقاطعة ويستتشستر (نيويورك)",
    "Geography of Westchester County, New York": "جغرافيا مقاطعة ويستتشستر (نيويورك)",
    "Images of Westchester County, New York": "صور من مقاطعة ويستتشستر (نيويورك)",
    "Landforms of Westchester County, New York": "تضاريس مقاطعة ويستتشستر (نيويورك)",
    "Languages of the Cayman Islands": "لغات جزر كايمان",
    "Olympic gold medalists for the United States": "فائزون بميداليات ذهبية أولمبية من الولايات المتحدة",
    "Olympic medalists for the United States": "فائزون بميداليات أولمبية من الولايات المتحدة",
    "Protected areas of Westchester County, New York": "مناطق محمية في مقاطعة ويستتشستر (نيويورك)",
}

geography_in_1 = {
    "Buildings and structures in the United States by state": "مبان ومنشآت في الولايات المتحدة حسب الولاية",
    "Buildings and structures in Westchester County, New York": "مبان ومنشآت في مقاطعة ويستتشستر (نيويورك)",
    "Cemeteries in Westchester County, New York": "مقابر في مقاطعة ويستتشستر (نيويورك)",
    "Centuries in the United States by state": "قرون في الولايات المتحدة حسب الولاية",
    "Christianity in Westchester County, New York": "المسيحية في مقاطعة ويستتشستر (نيويورك)",
    "Churches in Westchester County, New York": "كنائس في مقاطعة ويستتشستر (نيويورك)",
    "Communications in the United States by state": "الاتصالات في الولايات المتحدة حسب الولاية",
    "Companies based in Westchester County, New York": "شركات مقرها في مقاطعة ويستتشستر (نيويورك)",
    "Crime in Pennsylvania": "جريمة في بنسلفانيا",
    "Crimes in Pennsylvania": "جرائم في بنسلفانيا",
    "Crimes in the United States by state": "جرائم في الولايات المتحدة حسب الولاية",
    "Disasters in the United States by state": "كوارث في الولايات المتحدة حسب الولاية",
    "Education in the United States by state": "التعليم في الولايات المتحدة حسب الولاية",
    "Rail transport in Sri Lanka by province": "السكك الحديدية في سريلانكا حسب المقاطعة",
    "Riots and civil disorder in the United States by state": "شغب وعصيان مدني في الولايات المتحدة حسب الولاية",
    "Schools in Westchester County, New York": "مدارس في مقاطعة ويستتشستر (نيويورك)",
    "Science and technology in the United States by state": "العلوم والتقانة في الولايات المتحدة حسب الولاية",
    "Slavery in the United States by state": "العبودية في الولايات المتحدة حسب الولاية",
    "Sports venues in Westchester County, New York": "ملاعب رياضية في مقاطعة ويستتشستر (نيويورك)",
    "Education in Westchester County, New York": "التعليم في مقاطعة ويستتشستر (نيويورك)",
    "Films set in China by city": "أفلام تقع أحداثها في الصين حسب المدينة",
    "Films set in Westchester County, New York": "أفلام تقع أحداثها في مقاطعة ويستتشستر (نيويورك)",
    "Films shot in China by city": "أفلام مصورة في الصين حسب المدينة",
    "Forts in the United States by state": "حصون في الولايات المتحدة حسب الولاية",
    "Health in North Dakota": "الصحة في داكوتا الشمالية",
    "Health in the United States by state": "الصحة في الولايات المتحدة حسب الولاية",
}

geography_in_2 = {
    "Historic districts in Westchester County, New York": "المناطق التاريخية في مقاطعة ويستتشستر (نيويورك)",
    "Historic sites in the United States by state": "مواقع تاريخية في الولايات المتحدة حسب الولاية",
    "Historic trails and roads in the United States by state": "طرق وممرات تاريخية في الولايات المتحدة حسب الولاية",
    "Hospitals in Westchester County, New York": "مستشفيات في مقاطعة ويستتشستر (نيويورك)",
    "Houses in Westchester County, New York": "منازل في مقاطعة ويستتشستر (نيويورك)",
    "Landmarks in the United States by state": "معالم في الولايات المتحدة حسب الولاية",
    "Manufacturing in the United States by state": "تصنيع في الولايات المتحدة حسب الولاية",
    "Museums in Westchester County, New York": "متاحف في مقاطعة ويستتشستر (نيويورك)",
    "National Register of Historic Places in Westchester County, New York": "السجل الوطني للأماكن التاريخية في مقاطعة ويستتشستر (نيويورك)",
    "Nature reserves in the United States by state": "محميات طبيعية في الولايات المتحدة حسب الولاية",
    "Olympic gold medalists for the United States in alpine skiing": "فائزون بميداليات ذهبية أولمبية من الولايات المتحدة في التزلج على المنحدرات الثلجية",
    "Parks in Westchester County, New York": "متنزهات في مقاطعة ويستتشستر (نيويورك)",
    "People by state in the United States": "أشخاص حسب الولاية في الولايات المتحدة",
    "Populated places in Westchester County, New York": "أماكن مأهولة في مقاطعة ويستتشستر (نيويورك)",
    "Television shows set in Australia by city": "عروض تلفزيونية تقع أحداثها في أستراليا حسب المدينة",
    "Tourist attractions in the United States by state": "مواقع جذب سياحي في الولايات المتحدة حسب الولاية",
    "Tourist attractions in Westchester County, New York": "مواقع جذب سياحي في مقاطعة ويستتشستر (نيويورك)",
    "Transportation buildings and structures in Westchester County, New York": "مبان ومنشآت نقل في مقاطعة ويستتشستر (نيويورك)",
    "Transportation in the United States by state": "النقل في الولايات المتحدة حسب الولاية",
    "Transportation in Westchester County, New York": "النقل في مقاطعة ويستتشستر (نيويورك)",
    "Universities and colleges in Westchester County, New York": "جامعات وكليات في مقاطعة ويستتشستر (نيويورك)",
}

test_data = [
    ("test_geography", geography_data),
    ("test_geography_in_1", geography_in_1),
    ("test_geography_in_2", geography_in_2),
]


@pytest.mark.parametrize("category, expected", geography_data.items(), ids=geography_data.keys())
@pytest.mark.fast
def test_geography(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


@pytest.mark.parametrize("category, expected", geography_in_1.items(), ids=geography_in_1.keys())
@pytest.mark.fast
def test_geography_in_1(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


@pytest.mark.parametrize("category, expected", geography_in_2.items(), ids=geography_in_2.keys())
@pytest.mark.fast
def test_geography_in_2(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


@pytest.mark.dump
@pytest.mark.parametrize("name,data", test_data)
def test_dump_all(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_label_ar)
    dump_diff(diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



"""
LabsYearsFormat processing module.
"""

import logging
from typing import Callable, Optional

from ..time_formats.time_to_arabic import (
    convert_time_to_arabic,
    match_time_ar_first,
    match_time_en_first,
)

logger = logging.getLogger(__name__)


class MatchTimes:
    """Class for matching time patterns in text."""

    def __init__(self) -> None:
        """
        Initialize a MatchTimes instance.

        This initializer performs no setup and exists to allow subclasses to define their own initialization behavior.
        """

    def match_en_time(self, text: str) -> str:
        """Match English time in text."""
        result = match_time_en_first(text)
        logger.debug(f": result={result}")
        return result

    def match_ar_time(self, text: str) -> str:
        """Match Arabic time in text."""
        result = match_time_ar_first(text)
        logger.debug(f": result={result}")
        return result


class LabsYearsFormat(MatchTimes):
    def __init__(
        self,
        category_templates: dict[str, str],
        key_param_placeholder: str = "{year1}",
        value_param_placeholder: str = "{year1}",
        year_param_name: str = "year1",
        fixing_callback: Optional[Callable] = None,
    ) -> None:
        """Prepare reusable lookup tables for year-based category labels."""
        self.lookup_count = 0
        self.category_templates = category_templates
        self.year_param_name = year_param_name
        self.key_param_placeholder = key_param_placeholder
        self.value_param_placeholder = value_param_placeholder
        self.fixing_callback = fixing_callback

    def lab_from_year(self, category_r: str) -> tuple:
        """
        Given a string `category_r` representing a category, this function extracts the year from the category and returns a tuple containing the extracted year and the corresponding category key. If no year is found in the category, an empty string and an empty string are returned.

        Parameters:
        - `category_r` (str): The category from which to extract the year.

        Returns:
        - `tuple`: A tuple containing the extracted year and the corresponding category key. If no year is found, an empty string and an empty string are returned.
        """
        logger.debug(f"start : category_r={category_r}")
        from_year = ""
        cat_year = ""
        category_r = category_r.lower()
        year_match = self.match_en_time(category_r)
        logger.debug(f" matched year: year_match={year_match}")

        if not year_match:
            logger.debug(f"end : no year match found for {category_r}")
            return cat_year, from_year

        cat_year = year_match
        cat_key = category_r.replace(cat_year, self.key_param_placeholder).lower().replace("category:", "").strip()
        logger.debug(f" created key: cat_key={cat_key}")

        cat_year_ar = convert_time_to_arabic(cat_year)
        logger.debug(f" arabic year: cat_year_ar={cat_year_ar}")

        canonical_label = self.category_templates.get(cat_key)
        logger.debug(f" template lookup: canonical_label={canonical_label}")

        if canonical_label and self.value_param_placeholder in canonical_label and cat_year_ar:
            from_year = canonical_label.format_map({self.year_param_name: cat_year_ar})
            logger.debug(f" formatted: from_year={from_year}")

            if self.fixing_callback:
                from_year = self.fixing_callback(from_year)
                logger.debug(f" after callback: from_year={from_year}")

            self.lookup_count += 1
            logger.info(f"<<green>> : {self.lookup_count}, canonical_label={canonical_label}")
            logger.info(f"\t<<green>> category_r={category_r} , from_year={from_year}")

        logger.debug(f"end : category_r={category_r}, cat_year={cat_year}")
        return cat_year, from_year

    def lab_from_year_add(self, category_r: str, category_lab: str, en_year: str, ar_year: str = "") -> bool:
        """
        A function that converts the year in category_r and category_lab to self.key_param_placeholder and updates the category_templates dictionary accordingly.

        Parameters:
            category_r (str): The category from which to update the year.
            category_lab (str): The category from which to update the year.
            en_year (str): The English year to update in the categories.
            ar_year (str): The Arabic year (optional, derived if not provided).
        Returns:
            bool: True if the template was added successfully, False otherwise.
        """
        logger.debug(f"start : category_r={category_r}, category_lab={category_lab}")
        category_r = category_r.lower().replace("category:", "").strip()

        if not ar_year:
            category_lab_2 = category_lab.replace("بعقد ", "عقد ")
            ar_year = self.match_ar_time(category_lab_2)
            logger.debug(f" matched ar_year: ar_year={ar_year}")

        if not en_year:
            en_year = self.match_en_time(category_r)
            logger.debug(f" matched en_year: en_year={en_year}")

        if en_year.isdigit() and not ar_year:
            ar_year = en_year
            logger.debug(f" digit fallback: ar_year={ar_year}")

        if not ar_year or ar_year not in category_lab:
            logger.debug(f" failed: ar_year={ar_year} not in category_lab")
            return False

        if not en_year or en_year not in category_r:
            logger.debug(f" failed: en_year={en_year} not in category_r")
            return False

        cat_key = category_r.replace(en_year, self.key_param_placeholder)
        lab_key = category_lab.replace(ar_year, self.value_param_placeholder)

        logger.debug("<<yellow>> logic:")
        logger.debug(f"\t<<yellow>> cat_key={cat_key} , lab_key={lab_key}")

        self.category_templates[cat_key.lower()] = lab_key
        logger.debug(f"end : updated templates with {cat_key.lower()}")
        return True



#!/usr/bin/python3
"""
Integration tests for model_multi_data_double.py module.

This module provides integration tests for MultiDataFormatterDataDouble class
which combines FormatData with FormatDataDouble for double-key category translations.
"""

import pytest

from ArWikiCats.translations_formats.DataModel import FormatData
from ArWikiCats.translations_formats.DataModelDouble import FormatDataDouble
from ArWikiCats.translations_formats.DataModelDouble.model_multi_data_double import (
    MultiDataFormatterDataDouble,
)


class TestMultiDataFormatterDataDoubleIntegration:
    """Integration tests for MultiDataFormatterDataDouble class."""

    @pytest.fixture
    def film_bot(self):
        """Create a MultiDataFormatterDataDouble instance for film categories."""
        country_bot = FormatData(
            formatted_data={
                "{nat_en} films": "أفلام {nat_ar}",
                "{nat_en} {film_key} films": "أفلام {film_ar} {nat_ar}",
            },
            data_list={"british": "بريطانية", "american": "أمريكية", "french": "فرنسية"},
            key_placeholder="{nat_en}",
            value_placeholder="{nat_ar}",
        )
        genre_bot = FormatDataDouble(
            formatted_data={"{film_key} films": "أفلام {film_ar}"},
            data_list={
                "action": "حركة",
                "drama": "درامية",
                "comedy": "كوميدية",
                "horror": "رعب",
            },
            key_placeholder="{film_key}",
            value_placeholder="{film_ar}",
        )
        return MultiDataFormatterDataDouble(country_bot, genre_bot)

    def test_create_label_single_genre(self, film_bot):
        """Test create_label with single genre."""
        result = film_bot.create_label("british action films")
        assert result == "أفلام حركة بريطانية"

    def test_create_label_double_genre(self, film_bot):
        """Test create_label with double genre."""
        result = film_bot.create_label("american action drama films")
        assert result == "أفلام حركة درامية أمريكية"

    def test_create_label_nationality_only(self, film_bot):
        """Test create_label with nationality only."""
        result = film_bot.country_bot.search("french films")
        assert result == "أفلام فرنسية"

    def test_search_same_as_create_label(self, film_bot):
        """Test that search is same as create_label."""
        create_result = film_bot.create_label("british comedy films")
        search_result = film_bot.search("british comedy films")
        assert create_result == search_result

    def test_normalize_both_new(self, film_bot):
        """Test normalize_both_new extracts both keys."""
        result = film_bot.normalize_both_new("american action films")
        assert result.nat_key == "american"
        assert result.other_key == "action"
        assert result.category == "american action films"


class TestMultiDataFormatterDataDoubleSearchAll:
    """Integration tests for MultiDataFormatterDataDouble.search_all method."""

    @pytest.fixture
    def bot(self):
        """Create a MultiDataFormatterDataDouble instance."""
        country_bot = FormatData(
            formatted_data={
                "{nat_en} films": "أفلام {nat_ar}",
                "{nat_en} {film_key} films": "أفلام {film_ar} {nat_ar}",
            },
            data_list={"british": "بريطانية"},
            key_placeholder="{nat_en}",
            value_placeholder="{nat_ar}",
        )
        genre_bot = FormatDataDouble(
            formatted_data={"{film_key} films": "أفلام {film_ar}"},
            data_list={"action": "حركة", "drama": "درامية"},
            key_placeholder="{film_key}",
            value_placeholder="{film_ar}",
        )
        return MultiDataFormatterDataDouble(country_bot, genre_bot)

    def test_search_all_combined_label(self, bot):
        """Test search_all with combined nationality and genre."""
        result = bot.search_all("british action films")
        assert result == "أفلام حركة بريطانية"

    def test_search_all_country_only(self, bot):
        """Test search_all falls back to country_bot."""
        result = bot.search_all("british films")
        assert result == "أفلام بريطانية"

    def test_search_all_genre_only(self, bot):
        """Test search_all falls back to other_bot."""
        result = bot.search_all("action films")
        assert result == "أفلام حركة"

    def test_search_all_with_prefix(self, bot):
        """Test search_all with Arabic category prefix."""
        result = bot.search_all("Category:british action films", add_arabic_category_prefix=True)
        assert result == "تصنيف:أفلام حركة بريطانية"


class TestMultiDataFormatterDataDoubleWithDataToFind:
    """Integration tests for MultiDataFormatterDataDouble with data_to_find."""

    @pytest.fixture
    def bot_with_data_to_find(self):
        """Create a bot with data_to_find for direct lookups."""
        country_bot = FormatData(
            formatted_data={},
            data_list={"british": "بريطانية"},
            key_placeholder="{nat_en}",
            value_placeholder="{nat_ar}",
        )
        genre_bot = FormatDataDouble(
            formatted_data={},
            data_list={"action": "حركة"},
            key_placeholder="{film_key}",
            value_placeholder="{film_ar}",
        )
        data_to_find = {"special film": "فيلم خاص"}
        return MultiDataFormatterDataDouble(country_bot, genre_bot, data_to_find=data_to_find)

    def test_create_label_uses_data_to_find(self, bot_with_data_to_find):
        """Test that create_label uses data_to_find for direct lookups."""
        result = bot_with_data_to_find.create_label("special film")
        assert result == "فيلم خاص"


class TestMultiDataFormatterDataDoubleEdgeCases:
    """Integration tests for MultiDataFormatterDataDouble edge cases."""

    def test_empty_result_when_no_match(self):
        """Test returns empty string when no match found."""
        country_bot = FormatData(
            formatted_data={"{nat_en} films": "أفلام {nat_ar}"},
            data_list={"british": "بريطانية"},
            key_placeholder="{nat_en}",
            value_placeholder="{nat_ar}",
        )
        genre_bot = FormatDataDouble(
            formatted_data={"{film_key} films": "أفلام {film_ar}"},
            data_list={"action": "حركة"},
            key_placeholder="{film_key}",
            value_placeholder="{film_ar}",
        )
        bot = MultiDataFormatterDataDouble(country_bot, genre_bot)

        result = bot.create_label("unknown category")
        assert result == ""

    def test_double_key_genre(self):
        """Test with double-key genre like 'action drama'."""
        country_bot = FormatData(
            formatted_data={
                "{nat_en} {film_key} films": "أفلام {film_ar} {nat_ar}",
            },
            data_list={"british": "بريطانية"},
            key_placeholder="{nat_en}",
            value_placeholder="{nat_ar}",
        )
        genre_bot = FormatDataDouble(
            formatted_data={},
            data_list={
                "action": "حركة",
                "drama": "درامية",
            },
            key_placeholder="{film_key}",
            value_placeholder="{film_ar}",
        )
        bot = MultiDataFormatterDataDouble(country_bot, genre_bot)

        result = bot.create_label("british action drama films")
        assert result == "أفلام حركة درامية بريطانية"



""" """

import pytest

from ArWikiCats.legacy_bots.resolvers.arabic_label_builder import find_ar_label

data_list2 = [
    ("paralympic competitors for cyprus", " for ", "منافسون بارالمبيون من قبرص"),
    ("african games gold medalists for chad", " for ", "فائزون بميداليات ذهبية في الألعاب الإفريقية من تشاد"),
    ("olympic silver medalists for finland", " for ", "فائزون بميداليات فضية أولمبية من فنلندا"),
    ("summer olympics competitors for peru", " for ", "منافسون أولمبيون صيفيون من بيرو"),
]

test_data_in = {
    "1550 in belgian motorsport": "1550 في رياضة المحركات البلجيكية",
    "1550 in chinese motorsport": "1550 في رياضة المحركات الصينية",
    "1550 in english cricket": "1550 في الكريكت الإنجليزية",
    "1550 in french tennis": "1550 في كرة المضرب الفرنسية",
    "1550 in indian tennis": "1550 في كرة المضرب الهندية",
    "1550 in japanese football leagues": "1550 في دوريات كرة القدم اليابانية",
    "1550 in south american football leagues": "1550 في دوريات كرة القدم الأمريكية الجنوبية",
    "1550 in spanish tennis": "1550 في كرة المضرب الإسبانية",
    "1550 in swedish football leagues": "1550 في دوريات كرة القدم السويدية",
    "1550 disasters in australia": "كوارث 1550 في أستراليا",
    "1550 disasters in kenya": "كوارث 1550 في كينيا",
    "1550 disasters in mexico": "كوارث 1550 في المكسيك",
    "1550 disasters in oceania": "كوارث 1550 في أوقيانوسيا",
    "1550 in alabama": "1550 في ألاباما",
    "1550 in antarctica": "1550 في القارة القطبية الجنوبية",
    "1550 in antigua and barbuda": "1550 في أنتيغوا وباربودا",
    "1550 in archery": "1550 في النبالة",
    "1550 in australia": "1550 في أستراليا",
    "1550 in bermuda": "1550 في برمودا",
    "1550 in biotechnology": "1550 في تقانة حيوية",
    "1550 in brazilian sport": "1550 في رياضة برازيلية",
    "1550 in british columbia": "1550 في كولومبيا البريطانية",
    "1550 in british empire": "1550 في الإمبراطورية البريطانية",
    "1550 in british music": "1550 في موسيقى بريطانية",
    "1550 in bulgaria": "1550 في بلغاريا",
    "1550 in burmese sport": "1550 في رياضة بورمية",
    "1550 in cameroonian football": "1550 في كرة القدم الكاميرونية",
    "1550 in china": "1550 في الصين",
    "1550 in christianity": "1550 في المسيحية",
    "1550 in comics": "1550 في قصص مصورة",
    "1550 in danish television": "1550 في التلفزة الدنماركية",
    "1550 in delaware": "1550 في ديلاوير",
    "1550 in djibouti": "1550 في جيبوتي",
    "1550 in dominican republic": "1550 في جمهورية الدومينيكان",
    "1550 in economic history": "1550 في التاريخ الاقتصادي",
    "1550 in education": "1550 في تعليم",
    "1550 in europe": "1550 في أوروبا",
    "1550 in fencing": "1550 في مبارزة سيف الشيش",
    "1550 in field hockey": "1550 في هوكي الميدان",
    "1550 in finland": "1550 في فنلندا",
    "1550 in formula racing": "1550 في سباقات فورمولا",
    "1550 in french colonial empire": "1550 في الإمبراطورية الاستعمارية الفرنسية",
    "1550 in french television": "1550 في التلفزة الفرنسية",
    "1550 in georgia (u.s. state)": "1550 في ولاية جورجيا",
    "1550 in guam": "1550 في غوام",
    "1550 in guangzhou": "1550 في غوانزو",
    "1550 in habsburg monarchy": "1550 في ملكية هابسبورغ",
    "1550 in haiti": "1550 في هايتي",
    "1550 in holy roman empire": "1550 في الإمبراطورية الرومانية المقدسة",
    "1550 in horse racing": "1550 في سباق الخيل",
    "1550 in hungary": "1550 في المجر",
    "1550 in hurling": "1550 في القذف",
    "1550 in illinois": "1550 في إلينوي",
    "1550 in international sports": "1550 في رياضية دولية",
    "1550 in iraq": "1550 في العراق",
    "1550 in iraqi sport": "1550 في رياضة عراقية",
    "1550 in irish women's sport": "1550 في رياضة أيرلندية نسائية",
    "1550 in italian sport": "1550 في رياضة إيطالية",
    "1550 in ivory coast": "1550 في ساحل العاج",
    "1550 in khyber pakhtunkhwa": "1550 في خيبر بختونخوا",
    "1550 in law": "1550 في قانون",
    "1550 in liberia": "1550 في ليبيريا",
    "1550 in libyan sport": "1550 في رياضة ليبية",
    "1550 in lithuania": "1550 في ليتوانيا",
    "1550 in macau": "1550 في ماكاو",
    "1550 in maryland": "1550 في ماريلند",
    "1550 in mexican television": "1550 في التلفزة المكسيكية",
    "1550 in mexico": "1550 في المكسيك",
    "1550 in military history": "1550 في التاريخ العسكري",
    "1550 in minnesota": "1550 في منيسوتا",
    "1550 in missouri": "1550 في ميزوري",
    "1550 in mongolia": "1550 في منغوليا",
    "1550 in new jersey": "1550 في نيوجيرسي",
    "1550 in new zealand": "1550 في نيوزيلندا",
    "1550 in nordic combined": "1550 في التزلج النوردي المزدوج",
    "1550 in north america": "1550 في أمريكا الشمالية",
    "1550 in north macedonia": "1550 في مقدونيا الشمالية",
    "1550 in norwegian music": "1550 في موسيقى نرويجية",
    "1550 in oregon": "1550 في أوريغن",
    "1550 in outer space": "1550 في الفضاء الخارجي",
    "1550 in philippine cinema": "1550 في السينما الفلبينية",
    "1550 in professional wrestling": "1550 في مصارعة المحترفين",
    "1550 in puerto rico": "1550 في بورتوريكو",
    "1550 in religion": "1550 في الدين",
    "1550 in road cycling": "1550 في سباق الدراجات على الطريق",
    "1550 in romania": "1550 في رومانيا",
    "1550 in san marino": "1550 في سان مارينو",
    "1550 in sarawak": "1550 في ساراواك",
    "1550 in saudi arabia": "1550 في السعودية",
    "1550 in science": "1550 في العلم",
    "1550 in scotland": "1550 في إسكتلندا",
    "1550 in scottish women's sport": "1550 في رياضة إسكتلندية نسائية",
    "1550 in senegalese sport": "1550 في رياضة سنغالية",
    "1550 in serbia": "1550 في صربيا",
    "1550 in sint maarten": "1550 في سينت مارتن",
    "1550 in skeleton": "1550 في الزلاجات الصدرية",
    "1550 in somalian sport": "1550 في رياضة صومالية",
    "1550 in south america": "1550 في أمريكا الجنوبية",
    "1550 in spain": "1550 في إسبانيا",
    "1550 in spanish empire": "1550 في الإمبراطورية الإسبانية",
    "1550 in speed skating": "1550 في التزلج السريع",
    "1550 in syria": "1550 في سوريا",
    "1550 in taiwan": "1550 في تايوان",
    "1550 in taiwanese television": "1550 في التلفزة التايوانية",
    "1550 in tanzania": "1550 في تنزانيا",
    "1550 in thailand": "1550 في تايلاند",
    "1550 in toronto": "1550 في تورونتو",
    "1550 in united arab emirates": "1550 في الإمارات العربية المتحدة",
    "1550 in united nations": "1550 في الأمم المتحدة",
    "1550 in uruguayan football": "1550 في كرة القدم الأوروغويانية",
    "1550 in uzbekistani football": "1550 في كرة القدم الأوزبكستانية",
    "1550 in uzbekistani sport": "1550 في رياضة أوزبكستانية",
    "1550 in vanuatu": "1550 في فانواتو",
    "1550 in venezuela": "1550 في فنزويلا",
    "1550 in virginia": "1550 في فرجينيا",
    "1550 in washington (state)": "1550 في ولاية واشنطن",
    "1550 in west germany": "1550 في ألمانيا الغربية",
    "1550 in women's sport": "1550 في رياضة نسائية",
    "1550 mass shootings in africa": "إطلاق نار عشوائي 1550 في إفريقيا",
    "1550 mass shootings in oceania": "إطلاق نار عشوائي 1550 في أوقيانوسيا",
    "1550 murders in north america": "جرائم قتل في 1550 في أمريكا الشمالية",
    "1550 murders in russia": "جرائم قتل في 1550 في روسيا",
    "1550 natural disasters in united states": "كوارث طبيعية في 1550 في الولايات المتحدة",
    "1550s in asia": "عقد 1550 في آسيا",
    "1550s in belgium": "عقد 1550 في بلجيكا",
    "1550s in dundee": "عقد 1550 في دندي",
    "1550s in edmonton": "عقد 1550 في إدمونتون",
    "1550s in england": "عقد 1550 في إنجلترا",
    "1550s in estonia": "عقد 1550 في إستونيا",
    "1550s in hangzhou": "عقد 1550 في هانغتشو",
    "1550s in ice hockey": "عقد 1550 في هوكي الجليد",
    "1550s in japan": "عقد 1550 في اليابان",
    "1550s in literature": "عقد 1550 في أدب",
    "1550s in louisiana": "عقد 1550 في لويزيانا",
    "1550s in maltese sport": "عقد 1550 في رياضة مالطية",
    "1550s in north american sport": "عقد 1550 في رياضة أمريكية شمالية",
    "1550s in politics": "عقد 1550 في سياسة",
    "1550s in portugal": "عقد 1550 في البرتغال",
    "1550s in prince edward island": "عقد 1550 في جزيرة الأمير إدوارد",
    "1550s in quebec": "عقد 1550 في كيبك",
    "1550s in rugby union": "عقد 1550 في اتحاد الرجبي",
    "1550s in russia": "عقد 1550 في روسيا",
    "1550s in saudi arabia": "عقد 1550 في السعودية",
    "1550s in science": "عقد 1550 في العلم",
    "1550s in somaliland": "عقد 1550 في أرض الصومال",
    "1550s in south korean women's sport": "عقد 1550 في رياضة كورية جنوبية نسائية",
    "1550s in technology": "عقد 1550 في التقانة",
    "1550s in track cycling": "عقد 1550 في سباق الدراجات على المضمار",
    "1550s in ukrainian sport": "عقد 1550 في رياضة أوكرانية",
    "1550s in venezuela": "عقد 1550 في فنزويلا",
    "1550s in vietnam": "عقد 1550 في فيتنام",
    "1550s in winter sports": "عقد 1550 في الرياضة الشتوية",
    "20th century in iraq": "القرن 20 في العراق",
    "20th century in kent": "القرن 20 في كنت",
    "20th century in prince edward island": "القرن 20 في جزيرة الأمير إدوارد",
    "20th century in prussia": "القرن 20 في بروسيا",
    "20th century in quetta": "القرن 20 في كويته",
    "2nd millennium in kentucky": "الألفية 2 في كنتاكي",
    "3rd millennium in ghana": "الألفية 3 في غانا",
    "culture in lugano": "ثقافة في لوغانو",
    "education in caloocan": "تعليم في كالوكان",
    "transport in kirov oblast": "نقل في أوبلاست كيروف",
    "transport in newfoundland and labrador": "نقل في نيوفاوندلاند واللابرادور",
    "transport in oxford": "نقل في أكسفورد",
    "transport in ternopil oblast": "نقل في تيرنوبل أوبلاست",
}


@pytest.mark.parametrize("category, output", test_data_in.items(), ids=test_data_in.keys())
@pytest.mark.fast
def test_data_list_in_1(category: str, output: str) -> None:
    label = find_ar_label(category, " in ", use_event2=False)
    assert label == output


@pytest.mark.parametrize("category, separator, output", data_list2, ids=[x[0] for x in data_list2])
@pytest.mark.fast
def test_simple_2(category: str, separator: str, output: str) -> None:
    label = find_ar_label(category, separator, use_event2=False)
    assert label == output



"""
Resolver for job-related category labels with gender-specific logic.
This module provides functions to translate categories combining jobs,
nationalities, and genders into idiomatic Arabic.
"""

import functools
import logging

from ...translations import All_Nat
from ...translations_formats import MultiDataFormatterBaseV2, format_multi_data_v2
from .utils import fix_keys

logger = logging.getLogger(__name__)


def generate_jobs_data_dict() -> dict[str, dict[str, str]]:
    """
    Map English job identifiers to Arabic masculine, feminine, and inclusive labels.

    Each mapping value is a dictionary with keys `job_males`, `job_females`, and `both_jobs`, each holding the corresponding Arabic string for that job category.

    Returns:
        jobs_data (dict[str, dict[str, str]]): Mapping from an English job identifier (e.g., "actors") to a dict with Arabic forms under `job_males`, `job_females`, and `both_jobs`.
    """
    # "[ا-ي]+ون* و[ا-ي]+ات"
    jobs_data_new = {
        "bobsledders": {
            "job_males": "متزلجون جماعيون",
            "job_females": "متزلجات جماعيات",
            "both_jobs": "متزلجون ومتزلجات جماعيون",
        },
        "models": {"job_males": "عارضو أزياء", "job_females": "عارضات أزياء", "both_jobs": "عارضو وعارضات أزياء"},
        "actors": {"job_males": "ممثلون", "job_females": "ممثلات", "both_jobs": "ممثلون وممثلات"},
        "boxers": {"job_males": "ملاكمون", "job_females": "ملاكمات", "both_jobs": "ملاكمون وملاكمات"},
        "classical composers": {
            "job_males": "ملحنون كلاسيكيون",
            "job_females": "ملحنات كلاسيكيات",
            "both_jobs": "ملحنون وملحنات كلاسيكيون",
        },
        "composers": {"job_males": "ملحنون", "job_females": "ملحنات", "both_jobs": "ملحنون وملحنات"},
        "cyclists": {"job_males": "دراجون", "job_females": "دراجات", "both_jobs": "دراجون ودراجات"},
        "film actors": {"job_males": "ممثلو أفلام", "job_females": "ممثلات أفلام", "both_jobs": "ممثلو وممثلات أفلام"},
        "guitarists": {
            "job_males": "عازفو قيثارة",
            "job_females": "عازفات قيثارة",
            "both_jobs": "عازفو وعازفات قيثارة",
        },
        "singers": {"job_males": "مغنون", "job_females": "مغنيات", "both_jobs": "مغنون ومغنيات"},
        "television actors": {
            "job_males": "ممثلو تلفاز",
            "job_females": "ممثلات تلفاز",
            "both_jobs": "ممثلو وممثلات تلفاز",
        },
    }
    # TODO: Load from jobs_data_multi_one_word.json
    # jobs_data_new.update(open_json_file("jobs_data_multi_one_word.json"))

    return jobs_data_new


@functools.lru_cache(maxsize=1)
def generate_formatted_data() -> dict[str, str]:
    """
    Provide a mapping from English category patterns to Arabic template strings for gendered job and nationality phrases.

    Returns:
        dict[str, str]: Mapping where keys are English patterns containing placeholders (e.g., `{en_nat}`, `{job_en}`) and values are Arabic template strings using placeholders such as `{males}`, `{females}`, `{job_males}`, `{job_females}`, and `{both_jobs}`.
    """
    formatted_data = {
        "actresses": "ممثلات",
        "{en_nat} actresses": "ممثلات {females}",
        # [guitarists] = "عازفو وعازفات قيثارة"
        "{job_en}": "{both_jobs}",
        # [male guitarists] = "عازفو قيثارة"
        "male {job_en}": "{job_males}",
        # [american guitarists] = "عازفو وعازفات قيثارة أمريكيون"
        "{en_nat} {job_en}": "{both_jobs} {males}",
        # [american male guitarists] = "عازفو قيثارة أمريكيون"
        "{en_nat} male {job_en}": "{job_males} {males}",
        "male {en_nat} {job_en}": "{job_males} {males}",
        # [female guitarists] = "عازفات قيثارة"
        "female {job_en}": "{job_females}",
        # [american female guitarists] = "عازفات قيثارة أمريكيات"
        "{en_nat} female {job_en}": "{job_females} {females}",
        "female {en_nat} {job_en}": "{job_females} {females}",
        # test to add classical
        "classical {job_en}": "{both_jobs} كلاسيكيون",
        "male classical {job_en}": "{job_males} كلاسيكيون",
        "female classical {job_en}": "{job_females} كلاسيكيات",
        "{en_nat} classical {job_en}": "{both_jobs} كلاسيكيون {males}",
        "{en_nat} male classical {job_en}": "{job_males} كلاسيكيون {males}",
        "{en_nat} female classical {job_en}": "{job_females} كلاسيكيات {females}",
    }

    return formatted_data


@functools.lru_cache(maxsize=1)
def _job_bot() -> MultiDataFormatterBaseV2:
    jobs_data_new = generate_jobs_data_dict()

    nats_data = {
        x: {
            "males": v["males"],
            "females": v["females"],
        }
        for x, v in All_Nat.items()
        if v.get("males")
    }
    formatted_data = generate_formatted_data()

    return format_multi_data_v2(
        formatted_data=formatted_data,
        data_list=nats_data,
        key_placeholder="{en_nat}",
        data_list2=jobs_data_new,
        key2_placeholder="{job_en}",
        text_after="",
        text_before="the ",
        search_first_part=True,
        use_other_formatted_data=True,
    )


@functools.lru_cache(maxsize=10000)
def genders_jobs_resolver(category: str) -> str:
    normalized_category = fix_keys(category)
    logger.debug(f"<<yellow>> start {normalized_category=}")

    job_bot = _job_bot()
    result = job_bot.search_all_other_first(normalized_category)
    result = job_bot.prepend_arabic_category_prefix(category, result)

    logger.info(f"<<yellow>> end {category=}, {result=}")

    return result


__all__ = [
    "genders_jobs_resolver",
]



#!/usr/bin/python3
"""
Tests for model_data_double_v2.py module.

This module provides tests for FormatDataDoubleV2 class which handles
double-key template-driven category translations where data_list values
can be dictionaries.
"""

from ArWikiCats.translations_formats.DataModelDouble.model_data_double_v2 import (
    FormatDataDoubleV2,
)


class TestFormatDataDoubleV2Init:
    """Tests for FormatDataDoubleV2 initialization."""

    def test_init_default(self):
        """Test initialization with default parameters."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "action": {"genre_label": "أكشن"},
            "drama": {"genre_label": "دراما"},
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )
        assert bot.formatted_data == formatted_data
        assert bot.data_list == data_list
        assert bot.key_placeholder == "{genre}"
        assert bot.text_after == ""
        assert bot.text_before == ""
        assert bot.splitter == " "
        assert bot.ar_joiner == " "
        assert bot.sort_ar_labels is False
        assert bot.log_multi_cache is True

    def test_init_with_text_after(self):
        """Test initialization with text_after parameter."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
            text_after="test",
        )
        assert bot.text_after == "test"

    def test_init_with_text_before(self):
        """Test initialization with text_before parameter."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
            text_before="test ",
        )
        assert bot.text_before == "test "

    def test_init_with_splitter(self):
        """Test initialization with custom splitter."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
            splitter="[_ ]",
        )
        assert bot.splitter == "[_ ]"

    def test_init_with_ar_joiner(self):
        """Test initialization with custom ar_joiner."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
            ar_joiner=" و ",
        )
        assert bot.ar_joiner == " و "

    def test_init_with_sort_ar_labels(self):
        """Test initialization with sort_ar_labels=True."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
            sort_ar_labels=True,
        )
        assert bot.sort_ar_labels is True

    def test_init_with_log_multi_cache_false(self):
        """Test initialization with log_multi_cache=False."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
            log_multi_cache=False,
        )
        assert bot.log_multi_cache is False


class TestFormatDataDoubleV2UpdatePutLabelLast:
    """Tests for FormatDataDoubleV2.update_put_label_last method."""

    def test_update_put_label_last_with_list(self):
        """Test update_put_label_last with a list."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )
        bot.update_put_label_last(["upcoming", "recent"])
        assert bot.put_label_last == ["upcoming", "recent"]

    def test_update_put_label_last_with_set(self):
        """Test update_put_label_last with a set."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )
        bot.update_put_label_last({"upcoming", "recent"})
        assert bot.put_label_last == {"upcoming", "recent"}


class TestFormatDataDoubleV2Search:
    """Tests for FormatDataDoubleV2._search method."""

    def test_search_exact_match_in_formatted_data(self):
        """Test _search with exact match in formatted_data_ci."""
        formatted_data = {"action films": "أفلام أكشن"}
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )
        result = bot._search("action films")
        assert result == "أفلام أكشن"

    def test_search_single_key(self):
        """Test _search with single key match."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )
        result = bot._search("action films")
        assert result == "أفلام أكشن"

    def test_search_double_key(self):
        """Test _search with double key match."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "action": {"genre_label": "أكشن"},
            "drama": {"genre_label": "دراما"},
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )
        result = bot._search("action drama films")
        assert result == "أفلام أكشن دراما"

    def test_search_no_key_match(self):
        """Test _search when no key matches."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )
        result = bot._search("no match films")
        assert result == ""

    def test_search_no_label_match(self):
        """Test _search when key matches but no label found."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {}  # Empty data_list
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )
        result = bot._search("action films")
        assert result == ""

    def test_search_no_template_match(self):
        """Test _search when key and label match but no template found."""
        formatted_data = {}  # Empty formatted_data
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )
        result = bot._search("action films")
        assert result == ""


class TestFormatDataDoubleV2KeysToPatternDouble:
    """Tests for FormatDataDoubleV2.keys_to_pattern_double method."""

    def test_keys_to_pattern_double_returns_pattern(self):
        """Test keys_to_pattern_double returns a compiled regex pattern."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "action": {"genre_label": "أكشن"},
            "drama": {"genre_label": "دراما"},
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )
        pattern = bot.keys_to_pattern_double()
        assert pattern is not None
        # Should match "action drama" with space separator
        match = pattern.search(" action drama ")
        assert match is not None
        assert match.group(1) == "action"
        assert match.group(2) == " "
        assert match.group(3) == "drama"

    def test_keys_to_pattern_double_with_custom_splitter(self):
        """Test keys_to_pattern_double with custom splitter."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "action": {"genre_label": "أكشن"},
            "drama": {"genre_label": "دراما"},
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
            splitter="[_ ]",
        )
        pattern = bot.keys_to_pattern_double()
        assert pattern is not None
        # Should match "action_drama" with underscore separator
        match = pattern.search(" action_drama ")
        assert match is not None
        assert match.group(1) == "action"
        assert match.group(2) == "_"
        assert match.group(3) == "drama"

    def test_keys_to_pattern_double_empty_data_list(self):
        """Test keys_to_pattern_double returns None with empty data_list."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )
        pattern = bot.keys_to_pattern_double()
        assert pattern is None


class TestFormatDataDoubleV2MatchKey:
    """Tests for FormatDataDoubleV2.match_key method."""

    def test_match_key_exact_in_data_list(self):
        """Test match_key with exact match in data_list_ci."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )
        result = bot.match_key("action")
        assert result == "action"

    def test_match_key_double_key(self):
        """Test match_key with double key match."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "action": {"genre_label": "أكشن"},
            "drama": {"genre_label": "دراما"},
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )
        result = bot.match_key("action drama films")
        assert result == "action drama"
        # Check that keys_to_split was populated
        assert "action drama" in bot.keys_to_split
        assert bot.keys_to_split["action drama"] == ["action", "drama"]

    def test_match_key_single_key(self):
        """Test match_key with single key match."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )
        result = bot.match_key("action films")
        assert result == "action"

    def test_match_key_no_match(self):
        """Test match_key with no match."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )
        result = bot.match_key("no match here")
        assert result == ""

    def test_match_key_normalizes_spaces(self):
        """Test match_key normalizes extra spaces."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "action": {"genre_label": "أكشن"},
            "drama": {"genre_label": "دراما"},
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )
        result = bot.match_key("action    drama   films")
        assert result == "action drama"


class TestFormatDataDoubleV2ApplyPatternReplacement:
    """Tests for FormatDataDoubleV2.apply_pattern_replacement method."""

    def test_apply_pattern_replacement_dict(self):
        """Test apply_pattern_replacement with dict value."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن", "other": "other"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )

        template_label = "أفلام {genre_label}"
        sport_label = {"genre_label": "أكشن", "other": "other"}
        result = bot.apply_pattern_replacement(template_label, sport_label)
        assert result == "أفلام أكشن"

    def test_apply_pattern_replacement_string_value(self):
        """Test apply_pattern_replacement with string value returns template unchanged."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": "أكشن"}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )

        template_label = "أفلام {genre_label}"
        sport_label = "أكشن"
        result = bot.apply_pattern_replacement(template_label, sport_label)
        assert result == template_label

    def test_apply_pattern_replacement_multiple_placeholders(self):
        """Test apply_pattern_replacement with multiple placeholders."""
        formatted_data = {"{genre} films": "أفلام {genre_label} {other}"}
        data_list = {"action": {"genre_label": "أكشن", "other": "اخرى"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )

        template_label = "أفلام {genre_label} {other}"
        sport_label = {"genre_label": "أكشن", "other": "اخرى"}
        result = bot.apply_pattern_replacement(template_label, sport_label)
        assert result == "أفلام أكشن اخرى"


class TestFormatDataDoubleV2CreateLabelFromKeys:
    """Tests for FormatDataDoubleV2.create_label_from_keys method."""

    def test_create_label_from_keys_both_exist(self):
        """Test create_label_from_keys when both keys have dict labels."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "action": {"genre_label": "أكشن"},
            "drama": {"genre_label": "دراما"},
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )

        result = bot.create_label_from_keys("action", "drama")
        assert result == {"genre_label": "أكشن دراما"}

    def test_create_label_from_keys_missing_first(self):
        """Test create_label_from_keys when first key is missing."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"drama": {"genre_label": "دراما"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )

        result = bot.create_label_from_keys("action", "drama")
        assert result == ""

    def test_create_label_from_keys_missing_second(self):
        """Test create_label_from_keys when second key is missing."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )

        result = bot.create_label_from_keys("action", "drama")
        assert result == ""

    def test_create_label_from_keys_non_dict_first(self):
        """Test create_label_from_keys when first key has non-dict label."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "action": "أكشن",  # String instead of dict
            "drama": {"genre_label": "دراما"},
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )

        result = bot.create_label_from_keys("action", "drama")
        assert result == ""

    def test_create_label_from_keys_non_dict_second(self):
        """Test create_label_from_keys when second key has non-dict label."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "action": {"genre_label": "أكشن"},
            "drama": "دراما",  # String instead of dict
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )

        result = bot.create_label_from_keys("action", "drama")
        assert result == ""

    def test_create_label_from_keys_with_put_label_last(self):
        """Test create_label_from_keys with put_label_last."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "upcoming": {"genre_label": "قادمة"},
            "horror": {"genre_label": "رعب"},
            "yemeni": {"genre_label": "يمنية"},
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )
        bot.update_put_label_last(["upcoming"])

        result = bot.create_label_from_keys("upcoming", "horror")
        # "upcoming" is in put_label_last, so order is swapped
        assert result == {"genre_label": "رعب قادمة"}

    def test_create_label_from_keys_with_sort(self):
        """Test create_label_from_keys with sort_ar_labels=True."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "action": {"genre_label": "أكشن"},
            "drama": {"genre_label": "دراما"},
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
            sort_ar_labels=True,
        )

        result = bot.create_label_from_keys("action", "drama")
        # Should be sorted alphabetically: "أكشن" < "دراما"
        assert result == {"genre_label": "أكشن دراما"}

    def test_create_label_from_keys_with_log_multi_cache(self):
        """Test create_label_from_keys caches result when log_multi_cache=True."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "action": {"genre_label": "أكشن"},
            "drama": {"genre_label": "دراما"},
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
            log_multi_cache=True,
        )

        bot.create_label_from_keys("action", "drama")
        # Check that result was cached in search_multi_cache
        # Key is "drama action" (part2 + " " + part1)
        assert "drama action" in bot.search_multi_cache
        assert bot.search_multi_cache["drama action"] == {"genre_label": "أكشن دراما"}

    def test_create_label_from_keys_no_log_multi_cache(self):
        """Test create_label_from_keys doesn't cache when log_multi_cache=False."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "action": {"genre_label": "أكشن"},
            "drama": {"genre_label": "دراما"},
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
            log_multi_cache=False,
        )

        bot.create_label_from_keys("action", "drama")
        # Check that result was NOT cached in search_multi_cache
        assert "drama action" not in bot.search_multi_cache


class TestFormatDataDoubleV2GetKeyLabel:
    """Tests for FormatDataDoubleV2.get_key_label method."""

    def test_get_key_label_direct_match(self):
        """Test get_key_label with direct match in data_list_ci."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )

        result = bot.get_key_label("action")
        assert result == {"genre_label": "أكشن"}

    def test_get_key_label_from_cache(self):
        """Test get_key_label retrieves from search_multi_cache."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "action": {"genre_label": "أكشن"},
            "drama": {"genre_label": "دراما"},
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )

        # Pre-populate the cache
        bot.search_multi_cache["drama action"] = {"genre_label": "أكشن دراما"}

        result = bot.get_key_label("drama action")
        assert result == {"genre_label": "أكشن دراما"}

    def test_get_key_label_from_keys_to_split(self):
        """Test get_key_label creates label from keys_to_split."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "action": {"genre_label": "أكشن"},
            "drama": {"genre_label": "دراما"},
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )

        # Pre-populate keys_to_split
        bot.keys_to_split["action drama"] = ["action", "drama"]

        result = bot.get_key_label("action drama")
        assert result == {"genre_label": "أكشن دراما"}

    def test_get_key_label_no_match(self):
        """Test get_key_label returns empty string when no match."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )

        result = bot.get_key_label("nomatch")
        assert result == ""


class TestFormatDataDoubleV2ReplaceValuePlaceholder:
    """Tests for FormatDataDoubleV2.replace_value_placeholder method."""

    def test_replace_value_placeholder_dict(self):
        """Test replace_value_placeholder with dict value."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )

        label = "أفلام {genre_label}"
        value = {"genre_label": "أكشن"}
        result = bot.replace_value_placeholder(label, value)
        assert result == "أفلام أكشن"

    def test_replace_value_placeholder_multiple_keys(self):
        """Test replace_value_placeholder with multiple placeholder keys."""
        formatted_data = {"{genre} films": "أفلام {genre_label} {other}"}
        data_list = {"action": {"genre_label": "أكشن", "other": "اخرى"}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )

        label = "أفلام {genre_label} {other}"
        value = {"genre_label": "أكشن", "other": "اخرى"}
        result = bot.replace_value_placeholder(label, value)
        assert result == "أفلام أكشن اخرى"

    def test_replace_value_placeholder_string_value(self):
        """Test replace_value_placeholder with string value returns label unchanged."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": "أكشن"}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )

        label = "أفلام {genre_label}"
        value = "أكشن"
        result = bot.replace_value_placeholder(label, value)
        assert result == label

    def test_replace_value_placeholder_non_string_value_in_dict(self):
        """Test replace_value_placeholder skips non-string values in dict."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {"action": {"genre_label": "أكشن", "other": 123}}
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )

        label = "أفلام {genre_label} {other}"
        value = {"genre_label": "أكشن", "other": 123}
        result = bot.replace_value_placeholder(label, value)
        # Should replace genre_label but skip other (not a string)
        assert result == "أفلام أكشن {other}"


class TestFormatDataDoubleV2SearchIntegration:
    """Integration tests for FormatDataDoubleV2.search method."""

    def test_search_single_key_full(self):
        """Test full search flow with single key."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "action": {"genre_label": "أكشن"},
            "drama": {"genre_label": "دراما"},
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )

        result = bot.search("action films")
        assert result == "أفلام أكشن"

    def test_search_double_key_full(self):
        """Test full search flow with double key."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "action": {"genre_label": "أكشن"},
            "drama": {"genre_label": "دراما"},
            "comedy": {"genre_label": "كوميدي"},
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
        )

        result = bot.search("action drama films")
        assert result == "أفلام أكشن دراما"

    def test_search_with_custom_ar_joiner(self):
        """Test search with custom ar_joiner."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "action": {"genre_label": "أكشن"},
            "drama": {"genre_label": "دراما"},
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
            ar_joiner=" و ",
        )

        result = bot.search("action drama films")
        assert result == "أفلام أكشن و دراما"

    def test_search_with_sort_ar_labels(self):
        """Test search with sort_ar_labels=True."""
        formatted_data = {"{genre} films": "أفلام {genre_label}"}
        data_list = {
            "action": {"genre_label": "أكشن"},
            "drama": {"genre_label": "دراما"},
        }
        bot = FormatDataDoubleV2(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder="{genre}",
            sort_ar_labels=True,
        )

        result1 = bot.search("action drama films")
        result2 = bot.search("drama action films")
        # Both should be sorted the same way
        assert result1 == result2
        assert result1 == "أفلام أكشن دراما"
